@@ -538,43 +538,6 @@ static int in_evdev_menu_translate(void *drv_data, int keycode)
 	return 0;
 }
 
-/* FIXME: move to plat */
-#if 0
-static const struct {
-	short code;
-	char btype;
-	char bit;
-} in_evdev_def_binds[] =
-{
-	/* MXYZ SACB RLDU */
-	{ KEY_UP,	IN_BINDTYPE_PLAYER12, 0 },
-	{ KEY_DOWN,	IN_BINDTYPE_PLAYER12, 1 },
-	{ KEY_LEFT,	IN_BINDTYPE_PLAYER12, 2 },
-	{ KEY_RIGHT,	IN_BINDTYPE_PLAYER12, 3 },
-	{ KEY_S,	IN_BINDTYPE_PLAYER12, 4 },	/* B */
-	{ KEY_D,	IN_BINDTYPE_PLAYER12, 5 },	/* C */
-	{ KEY_A,	IN_BINDTYPE_PLAYER12, 6 },	/* A */
-	{ KEY_ENTER,	IN_BINDTYPE_PLAYER12, 7 },
-	{ KEY_BACKSLASH, IN_BINDTYPE_EMU, PEVB_MENU },
-	/* Pandora */
-	{ KEY_PAGEDOWN,	IN_BINDTYPE_PLAYER12, 4 },
-	{ KEY_END,	IN_BINDTYPE_PLAYER12, 5 },
-	{ KEY_HOME,	IN_BINDTYPE_PLAYER12, 6 },
-	{ KEY_LEFTALT,	IN_BINDTYPE_PLAYER12, 7 },
-	{ KEY_RIGHTSHIFT,IN_BINDTYPE_EMU, PEVB_STATE_SAVE },
-	{ KEY_RIGHTCTRL, IN_BINDTYPE_EMU, PEVB_STATE_LOAD },
-	{ KEY_LEFTCTRL,	 IN_BINDTYPE_EMU, PEVB_MENU },
-	/* Caanoo */
-	{ BTN_THUMB,	IN_BINDTYPE_PLAYER12, 4 },	/* B */
-	{ BTN_THUMB2,	IN_BINDTYPE_PLAYER12, 5 },	/* C */
-	{ BTN_TRIGGER,	IN_BINDTYPE_PLAYER12, 6 },	/* A */
-	{ BTN_BASE3,	IN_BINDTYPE_PLAYER12, 7 },
-	{ BTN_TOP2,	IN_BINDTYPE_EMU, PEVB_STATE_SAVE },
-	{ BTN_PINKIE,	IN_BINDTYPE_EMU, PEVB_STATE_LOAD },
-	{ BTN_BASE,	IN_BINDTYPE_EMU, PEVB_MENU },
-};
-#endif
-
 static void in_evdev_get_def_binds(int *binds)
 {
 	int i;
