@@ -27,6 +27,10 @@
 #include "common/input.h"
 #include "common/readpng.h"
 
+// don't include debug.h - it breaks ARM build (R1 redefined)
+void StartDebugger();
+void StopDebugger();
+
 int ready_to_go;
 unsigned long gpuDisp;
 char cfgfile_basename[MAXPATHLEN];
@@ -36,9 +40,6 @@ enum sched_action emu_action, emu_action_old;
 char hud_msg[64];
 int hud_new_msg;
 
-// from softgpu plugin
-extern int UseFrameSkip;
-
 static void make_path(char *buf, size_t size, const char *dir, const char *fname)
 {
 	if (fname)
@@ -97,7 +98,7 @@ void set_cd_image(const char *fname)
 
 	if (ext && (
 	    strcasecmp(ext, ".z") == 0 || strcasecmp(ext, ".bz") == 0 ||
-	    strcasecmp(ext, ".znx") == 0 || strcasecmp(ext, ".pbp") == 0)) {
+	    strcasecmp(ext, ".znx") == 0 /*|| strcasecmp(ext, ".pbp") == 0*/)) {
 		SetIsoFile(NULL);
 		cdrcimg_set_fname(fname);
 		strcpy(Config.Cdr, "builtin_cdrcimg");
@@ -120,11 +121,33 @@ static void set_default_paths(void)
 	strcpy(Config.Pad1, "builtin_pad");
 	strcpy(Config.Pad2, "builtin_pad");
 	strcpy(Config.Net, "Disabled");
+#if defined(__arm__) && !defined(__ARM_ARCH_7A__) /* XXX */
+	strcpy(Config.Gpu, "gpuPCSX4ALL.so");
+#endif
 	Config.PsxAuto = 1;
 
 	snprintf(Config.PatchesDir, sizeof(Config.PatchesDir), "." PATCHES_DIR);
 }
 
+static void check_memcards(void)
+{
+	char buf[MAXPATHLEN];
+	FILE *f;
+	int i;
+
+	for (i = 1; i <= 9; i++) {
+		snprintf(buf, sizeof(buf), ".%scard%d.mcd", MEMCARD_DIR, i);
+
+		f = fopen(buf, "rb");
+		if (f == NULL) {
+			printf("Creating memcard: %s\n", buf);
+			CreateMcd(buf);
+		}
+		else
+			fclose(f);
+	}
+}
+
 void do_emu_action(void)
 {
 	char buf[MAXPATHLEN];
@@ -133,8 +156,6 @@ void do_emu_action(void)
 	emu_action_old = emu_action;
 
 	switch (emu_action) {
-	case SACTION_NONE:
-		return;
 	case SACTION_ENTER_MENU:
 		menu_loop();
 		return;
@@ -157,9 +178,14 @@ void do_emu_action(void)
 			state_slot = 9;
 		goto do_state_slot;
 	case SACTION_TOGGLE_FSKIP:
-		UseFrameSkip ^= 1;
-		snprintf(hud_msg, sizeof(hud_msg), "FRAMESKIP %s",
-			UseFrameSkip ? "ON" : "OFF");
+		pl_rearmed_cbs.fskip_advice = 0;
+		pl_rearmed_cbs.frameskip++;
+		if (pl_rearmed_cbs.frameskip > 1)
+			pl_rearmed_cbs.frameskip = -1;
+		snprintf(hud_msg, sizeof(hud_msg), "FRAMESKIP: %s",
+			pl_rearmed_cbs.frameskip == -1 ? "AUTO" :
+			pl_rearmed_cbs.frameskip == 0 ? "OFF" : "1" );
+		plugin_call_rearmed_cbs();
 		break;
 	case SACTION_SCREENSHOT:
 		{
@@ -180,6 +206,8 @@ void do_emu_action(void)
 				snprintf(hud_msg, sizeof(hud_msg), "SCREENSHOT TAKEN");
 			break;
 		}
+	default:
+		return;
 	}
 	hud_new_msg = 3;
 	return;
@@ -188,6 +216,7 @@ do_state_slot:
 	snprintf(hud_msg, sizeof(hud_msg), "STATE SLOT %d [%s]", state_slot,
 		emu_check_state(state_slot) == 0 ? "USED" : "FREE");
 	hud_new_msg = 3;
+	printf("* %s\n", hud_msg);
 }
 
 int main(int argc, char *argv[])
@@ -216,6 +245,7 @@ int main(int argc, char *argv[])
 
 	CheckSubDir();
 	set_default_paths();
+	check_memcards();
 	strcpy(Config.Bios, "HLE");
 
 #ifdef MAEMO
@@ -225,12 +255,14 @@ int main(int argc, char *argv[])
 	char file[MAXPATHLEN] = "";
 	char path[MAXPATHLEN];
 	const char *cdfile = NULL;
+	const char *loadst_f = NULL;
+	int psxout = 0;
 	int loadst = 0;
 	int i;
 
 	// read command line options
 	for (i = 1; i < argc; i++) {
-		     if (!strcmp(argv[i], "-psxout")) Config.PsxOut = 1;
+		     if (!strcmp(argv[i], "-psxout")) psxout = 1;
 		else if (!strcmp(argv[i], "-load")) loadst = atol(argv[++i]);
 		else if (!strcmp(argv[i], "-cfg")) {
 			if (i+1 >= argc) break;
@@ -254,6 +286,10 @@ int main(int argc, char *argv[])
 
 			cdfile = isofilename;
 		}
+		else if (!strcmp(argv[i], "-loadf")) {
+			if (i+1 >= argc) break;
+			loadst_f = argv[++i];
+		}
 		else if (!strcmp(argv[i], "-h") ||
 			 !strcmp(argv[i], "-help") ||
 			 !strcmp(argv[i], "--help")) {
@@ -293,7 +329,11 @@ int main(int argc, char *argv[])
 	in_init();
 	//in_probe();
 	plat_init();
-	menu_init();
+	menu_init(); // loads config
+	pl_init();
+
+	if (psxout)
+		Config.PsxOut = 1;
 
 	if (LoadPlugins() == -1) {
 		// FIXME: this recovery doesn't work, just delete bad config and bail out
@@ -336,6 +376,10 @@ int main(int argc, char *argv[])
 			int ret = emu_load_state(loadst - 1);
 			printf("%s state %d\n", ret ? "failed to load" : "loaded", loadst);
 		}
+		if (loadst_f) {
+			int ret = LoadState(loadst_f);
+			printf("%s state file: %s\n", ret ? "failed to load" : "loaded", loadst_f);
+		}
 	}
 	else
 		menu_loop();
@@ -362,7 +406,7 @@ int SysInit() {
 		return -1;
 	}
 
-	LoadMcds(Config.Mcd1, Config.Mcd2);	/* TODO Do we need to have this here, or in the calling main() function?? */
+	LoadMcds(Config.Mcd1, Config.Mcd2);
 
 	if (Config.Debug) {
 		StartDebugger();
@@ -446,7 +490,9 @@ int emu_save_state(int slot)
 	if (ret != 0)
 		return ret;
 
-	return SaveState(fname);
+	ret = SaveState(fname);
+	printf("* %s \"%s\" [%d]\n", ret == 0 ? "saved" : "failed to save", fname, slot);
+	return ret;
 }
 
 int emu_load_state(int slot)
@@ -457,8 +503,6 @@ int emu_load_state(int slot)
 	ret = get_state_filename(fname, sizeof(fname), slot);
 	if (ret != 0)
 		return ret;
-	
-
 
 	return LoadState(fname);
 }
