@@ -20,9 +20,9 @@
 #include "plugin.h"
 #include "plugin_lib.h"
 #include "omap.h"
-#include "pandora.h"
+#include "plat.h"
 #include "pcnt.h"
-#include "arm_utils.h"
+#include "cspace.h"
 #include "common/plat.h"
 #include "common/input.h"
 #include "linux/in_evdev.h"
@@ -30,10 +30,9 @@
 #include "../libpcsxcore/cdrom.h"
 #include "../libpcsxcore/psemu_plugin_defs.h"
 #include "../libpcsxcore/new_dynarec/new_dynarec.h"
-#include "../plugins/dfinput/pad.h"
+#include "../plugins/dfinput/main.h"
 #include "revision.h"
 
-#define MENU_X2 1
 #define array_size(x) (sizeof(x) / sizeof(x[0]))
 
 typedef enum
@@ -46,6 +45,7 @@ typedef enum
 	MA_MAIN_LOAD_ROM,
 	MA_MAIN_SWAP_CD,
 	MA_MAIN_RUN_BIOS,
+	MA_MAIN_RUN_EXE,
 	MA_MAIN_CONTROLS,
 	MA_MAIN_CREDITS,
 	MA_MAIN_EXIT,
@@ -54,33 +54,40 @@ typedef enum
 	MA_CTRL_EMU,
 	MA_CTRL_DEV_FIRST,
 	MA_CTRL_DEV_NEXT,
+	MA_CTRL_NUBS_BTNS,
+	MA_CTRL_DEADZONE,
 	MA_CTRL_DONE,
 	MA_OPT_SAVECFG,
 	MA_OPT_SAVECFG_GAME,
 	MA_OPT_CPU_CLOCKS,
 	MA_OPT_FILTERING,
+	MA_OPT_DISP_OPTS,
 } menu_id;
 
 enum {
 	SCALE_1_1,
 	SCALE_4_3,
+	SCALE_4_3v2,
 	SCALE_FULLSCREEN,
 	SCALE_CUSTOM,
 };
 
 static int last_psx_w, last_psx_h, last_psx_bpp;
-static int scaling, filter, cpu_clock, cpu_clock_st;
+static int scaling, filter, cpu_clock, cpu_clock_st, volume_boost, frameskip;
 static char rom_fname_reload[MAXPATHLEN];
 static char last_selected_fname[MAXPATHLEN];
-static int warned_about_bios, region, in_type_sel;
-int g_opts;
+static int warned_about_bios, region, in_type_sel1, in_type_sel2;
+static int psx_clock;
+static int memcard1_sel, memcard2_sel;
+int g_opts, analog_deadzone;
 
-// from softgpu plugin
-extern int iUseDither;
-extern int UseFrameSkip;
-extern uint32_t dwActFixes;
-extern float fFrameRateHz;
-extern int dwFrameRateTicks;
+#ifdef __ARM_ARCH_7A__
+#define DEFAULT_PSX_CLOCK 57
+#define DEFAULT_PSX_CLOCK_S "57"
+#else
+#define DEFAULT_PSX_CLOCK 50
+#define DEFAULT_PSX_CLOCK_S "50"
+#endif
 
 // sound plugin
 extern int iUseReverb;
@@ -88,10 +95,12 @@ extern int iUseInterpolation;
 extern int iXAPitch;
 extern int iSPUIRQWait;
 extern int iUseTimer;
+extern int iVolume;
 
 static const char *bioses[24];
 static const char *gpu_plugins[16];
 static const char *spu_plugins[16];
+static const char *memcards[32];
 static int bios_sel, gpu_plugsel, spu_plugsel;
 
 
@@ -146,38 +155,59 @@ static void menu_sync_config(void)
 		Config.PsxAuto = 0;
 		Config.PsxType = region - 1;
 	}
-	in_type = in_type_sel ? PSE_PAD_TYPE_ANALOGPAD : PSE_PAD_TYPE_STANDARD;
+	cycle_multiplier = 10000 / psx_clock;
+
+	switch (in_type_sel1) {
+	case 1:  in_type1 = PSE_PAD_TYPE_ANALOGPAD; break;
+	case 2:  in_type1 = PSE_PAD_TYPE_GUNCON;    break;
+	default: in_type1 = PSE_PAD_TYPE_STANDARD;
+	}
+	switch (in_type_sel2) {
+	case 1:  in_type2 = PSE_PAD_TYPE_ANALOGPAD; break;
+	case 2:  in_type2 = PSE_PAD_TYPE_GUNCON;    break;
+	default: in_type2 = PSE_PAD_TYPE_STANDARD;
+	}
 	if (in_evdev_allow_abs_only != allow_abs_only_old) {
-		pandora_rescan_inputs();
+		plat_rescan_inputs();
 		allow_abs_only_old = in_evdev_allow_abs_only;
 	}
 
-	pl_frame_interval = Config.PsxType ? 20000 : 16667;
-	// used by P.E.Op.S. frameskip code
-	fFrameRateHz = Config.PsxType ? 50.0f : 59.94f;
-	dwFrameRateTicks = (100000*100 / (unsigned long)(fFrameRateHz*100));
+	iVolume = 768 + 128 * volume_boost;
+	pl_rearmed_cbs.frameskip = frameskip - 1;
+	pl_timing_prepare(Config.PsxType);
 }
 
 static void menu_set_defconfig(void)
 {
 	g_opts = 0;
 	scaling = SCALE_4_3;
+	volume_boost = 0;
+	frameskip = 0;
+	analog_deadzone = 70;
+	psx_clock = DEFAULT_PSX_CLOCK;
 
 	region = 0;
-	in_type_sel = 0;
+	in_type_sel1 = in_type_sel2 = 0;
 	in_evdev_allow_abs_only = 0;
 	Config.Xa = Config.Cdda = Config.Sio =
 	Config.SpuIrq = Config.RCntFix = Config.VSyncWA = 0;
+	Config.CdrReschedule = 0;
 
-	iUseDither = 0;
-	UseFrameSkip = 1;
-	dwActFixes = 1<<7;
+	pl_rearmed_cbs.gpu_peops.iUseDither = 0;
+	pl_rearmed_cbs.gpu_peops.dwActFixes = 1<<7;
+	pl_rearmed_cbs.gpu_unai.abe_hack =
+	pl_rearmed_cbs.gpu_unai.no_light =
+	pl_rearmed_cbs.gpu_unai.no_blend = 0;
 
 	iUseReverb = 2;
 	iUseInterpolation = 1;
 	iXAPitch = 0;
 	iSPUIRQWait = 1;
 	iUseTimer = 2;
+#ifndef __ARM_ARCH_7A__ /* XXX */
+	iUseReverb = 0;
+	iUseInterpolation = 0;
+#endif
 
 	menu_sync_config();
 }
@@ -194,10 +224,16 @@ static void menu_set_defconfig(void)
 #define CE_INTVAL(val) \
 	{ #val, sizeof(val), &val }
 
+#define CE_INTVAL_P(val) \
+	{ #val, sizeof(pl_rearmed_cbs.val), &pl_rearmed_cbs.val }
+
 // 'versioned' var, used when defaults change
 #define CE_INTVAL_V(val, ver) \
 	{ #val #ver, sizeof(val), &val }
 
+#define CE_INTVAL_PV(val, ver) \
+	{ #val #ver, sizeof(pl_rearmed_cbs.val), &pl_rearmed_cbs.val }
+
 static const struct {
 	const char *name;
 	size_t len;
@@ -217,8 +253,9 @@ static const struct {
 	CE_CONFIG_VAL(RCntFix),
 	CE_CONFIG_VAL(VSyncWA),
 	CE_CONFIG_VAL(Cpu),
+	CE_CONFIG_VAL(CdrReschedule),
 	CE_INTVAL(region),
-	CE_INTVAL(scaling),
+	CE_INTVAL_V(scaling, 2),
 	CE_INTVAL(g_layer_x),
 	CE_INTVAL(g_layer_y),
 	CE_INTVAL(g_layer_w),
@@ -227,17 +264,24 @@ static const struct {
 	CE_INTVAL(state_slot),
 	CE_INTVAL(cpu_clock),
 	CE_INTVAL(g_opts),
-	CE_INTVAL(in_type_sel),
-	CE_INTVAL(iUseDither),
-	CE_INTVAL(UseFrameSkip),
-	CE_INTVAL(dwActFixes),
-	CE_INTVAL(iUseReverb),
-	CE_INTVAL(iXAPitch),
-	CE_INTVAL_V(iUseInterpolation, 2),
-	CE_INTVAL_V(iSPUIRQWait, 2),
-	CE_INTVAL(iUseTimer),
+	CE_INTVAL(in_type_sel1),
+	CE_INTVAL(in_type_sel2),
+	CE_INTVAL(analog_deadzone),
+	CE_INTVAL_V(frameskip, 2),
+	CE_INTVAL_P(gpu_peops.iUseDither),
+	CE_INTVAL_P(gpu_peops.dwActFixes),
+	CE_INTVAL_P(gpu_unai.abe_hack),
+	CE_INTVAL_P(gpu_unai.no_light),
+	CE_INTVAL_P(gpu_unai.no_blend),
+	CE_INTVAL_V(iUseReverb, 3),
+	CE_INTVAL_V(iXAPitch, 3),
+	CE_INTVAL_V(iUseInterpolation, 3),
+	CE_INTVAL_V(iSPUIRQWait, 3),
+	CE_INTVAL_V(iUseTimer, 3),
 	CE_INTVAL(warned_about_bios),
 	CE_INTVAL(in_evdev_allow_abs_only),
+	CE_INTVAL(volume_boost),
+	CE_INTVAL(psx_clock),
 };
 
 static char *get_cd_label(void)
@@ -334,7 +378,7 @@ static int menu_load_config(int is_game)
 	f = fopen(cfgfile, "r");
 	if (f == NULL) {
 		printf("menu_load_config: failed to open: %s\n", cfgfile);
-		return -1;
+		goto fail;
 	}
 
 	fseek(f, 0, SEEK_END);
@@ -402,6 +446,14 @@ static int menu_load_config(int is_game)
 		}
 	}
 
+	keys_load_all(cfg);
+	ret = 0;
+fail_read:
+	free(cfg);
+fail:
+	if (f != NULL)
+		fclose(f);
+
 	menu_sync_config();
 
 	// sync plugins
@@ -417,19 +469,13 @@ static int menu_load_config(int is_game)
 		if (strcmp(Config.Spu, spu_plugins[i]) == 0)
 			{ spu_plugsel = i; break; }
 
-	keys_load_all(cfg);
-	ret = 0;
-fail_read:
-	free(cfg);
-fail:
-	fclose(f);
 	return ret;
 }
 
 // rrrr rggg gggb bbbb
 static unsigned short fname2color(const char *fname)
 {
-	static const char *cdimg_exts[] = { ".bin", ".img", ".iso", ".cue", ".z", ".bz", ".znx", ".pbp" };
+	static const char *cdimg_exts[] = { ".bin", ".img", ".mdf", ".iso", ".cue", ".z", ".bz", ".znx", ".pbp" };
 	static const char *other_exts[] = { ".ccd", ".toc", ".mds", ".sub", ".table", ".index", ".sbi" };
 	const char *ext = strrchr(fname, '.');
 	int i;
@@ -447,7 +493,17 @@ static unsigned short fname2color(const char *fname)
 
 static void draw_savestate_bg(int slot);
 
+static const char *filter_exts[] = {
+	".mp3", ".MP3", ".txt", ".htm", "html",	".jpg", ".pnd"
+};
+
 #define MENU_ALIGN_LEFT
+#ifdef __ARM_ARCH_7A__ // assume hires device
+#define MENU_X2 1
+#else
+#define MENU_X2 0
+#endif
+
 #define menu_init menu_init_common
 #include "common/menu.c"
 #undef menu_init
@@ -493,12 +549,12 @@ static void draw_savestate_bg(int slot)
 
 	memcpy(g_menubg_ptr, g_menubg_src_ptr, g_menuscreen_w * g_menuscreen_h * 2);
 
-	if ((gpu->ulStatus & 0x800000) || (gpu->ulStatus & 0x200000))
-		goto out; // disabled || 24bpp (NYET)
+	if (gpu->ulStatus & 0x800000)
+		goto out; // disabled
 
 	x = gpu->ulControl[5] & 0x3ff;
 	y = (gpu->ulControl[5] >> 10) & 0x1ff;
-	s = (u16 *)gpu->psxVRam + y * 1024 + (x & ~3);
+	s = (u16 *)gpu->psxVRam + y * 1024 + (x & ~1);
 	w = psx_widths[(gpu->ulStatus >> 16) & 7];
 	tmp = gpu->ulControl[7];
 	h = ((tmp >> 10) & 0x3ff) - (tmp & 0x3ff);
@@ -511,41 +567,26 @@ static void draw_savestate_bg(int slot)
 	h = min(g_menuscreen_h, h);
 	d = (u16 *)g_menubg_ptr + g_menuscreen_w * y + x;
 
-	for (; h > 0; h--, d += g_menuscreen_w, s += 1024)
-		bgr555_to_rgb565(d, s, w * 2);
+	for (; h > 0; h--, d += g_menuscreen_w, s += 1024) {
+		if (gpu->ulStatus & 0x200000)
+			bgr888_to_rgb565(d, s, w * 3);
+		else
+			bgr555_to_rgb565(d, s, w * 2);
+#ifndef __ARM_ARCH_7A__
+		// better darken this on small screens
+		menu_darken_bg(d, d, w * 2, 0);
+#endif
+	}
 
 out:
 	free(gpu);
 }
 
-// ---------- pandora specific -----------
+// ---------- XXX: pandora specific -----------
 
 static const char pnd_script_base[] = "sudo -n /usr/pandora/scripts";
 static char **pnd_filter_list;
 
-static int get_cpu_clock(void)
-{
-	FILE *f;
-	int ret = 0;
-	f = fopen("/proc/pandora/cpu_mhz_max", "r");
-	if (f) {
-		fscanf(f, "%d", &ret);
-		fclose(f);
-	}
-	return ret;
-}
-
-static void apply_cpu_clock(void)
-{
-	char buf[128];
-
-	if (cpu_clock != 0 && cpu_clock != get_cpu_clock()) {
-		snprintf(buf, sizeof(buf), "unset DISPLAY; echo y | %s/op_cpuspeed.sh %d",
-			 pnd_script_base, cpu_clock);
-		system(buf);
-	}
-}
-
 static void apply_filter(int which)
 {
 	static int old = -1;
@@ -591,7 +632,7 @@ static void pnd_menu_init(void)
 	char buff[64];
 	DIR *dir;
 
-	cpu_clock_st = cpu_clock = get_cpu_clock();
+	cpu_clock_st = cpu_clock = plat_cpu_clock_get();
 
 	dir = opendir("/etc/pandora/conf/dss_fir");
 	if (dir == NULL) {
@@ -658,8 +699,7 @@ static void pnd_menu_init(void)
 
 void menu_finish(void)
 {
-	cpu_clock = cpu_clock_st;
-	apply_cpu_clock();
+	plat_cpu_clock_apply(cpu_clock_st);
 }
 
 // -------------- key config --------------
@@ -694,6 +734,10 @@ me_bind_action emuctrl_actions[] =
 	{ "Toggle Frameskip ", 1 << SACTION_TOGGLE_FSKIP },
 	{ "Take Screenshot  ", 1 << SACTION_SCREENSHOT },
 	{ "Enter Menu       ", 1 << SACTION_ENTER_MENU },
+	{ "Gun Trigger      ", 1 << SACTION_GUN_TRIGGER },
+	{ "Gun A button     ", 1 << SACTION_GUN_A },
+	{ "Gun B button     ", 1 << SACTION_GUN_B },
+	{ "Gun Offscreen Trigger", 1 << SACTION_GUN_TRIGGER2 },
 	{ NULL,                0 }
 };
 
@@ -923,23 +967,32 @@ static int mh_savecfg(int id, int keys)
 static int mh_input_rescan(int id, int keys)
 {
 	//menu_sync_config();
-	pandora_rescan_inputs();
+	plat_rescan_inputs();
 	me_update_msg("rescan complete.");
 
 	return 0;
 }
 
-static const char *men_in_type_sel[] = { "Standard (SCPH-1080)", "Analog (SCPH-1150)", NULL };
+static const char *men_in_type_sel[] = {
+	"Standard (SCPH-1080)",
+	"Analog (SCPH-1150)",
+	"GunCon",
+	NULL
+};
 static const char h_nub_btns[] = "Experimental, keep this OFF if unsure. Select rescan after change.";
+static const char h_notsgun[] =  "Don't trigger (shoot) when touching screen in gun games.";
 
 static menu_entry e_menu_keyconfig[] =
 {
-	mee_handler_id("Player 1",          MA_CTRL_PLAYER1,    key_config_loop_wrap),
-	mee_handler_id("Player 2",          MA_CTRL_PLAYER2,    key_config_loop_wrap),
-	mee_handler_id("Emulator controls", MA_CTRL_EMU,        key_config_loop_wrap),
+	mee_handler_id("Player 1",              MA_CTRL_PLAYER1,    key_config_loop_wrap),
+	mee_handler_id("Player 2",              MA_CTRL_PLAYER2,    key_config_loop_wrap),
+	mee_handler_id("Emulator/Gun controls", MA_CTRL_EMU,        key_config_loop_wrap),
 	mee_label     (""),
-	mee_enum      ("Controller",        0, in_type_sel,     men_in_type_sel),
-	mee_onoff_h   ("Nubs as buttons",   0, in_evdev_allow_abs_only, 1, h_nub_btns),
+	mee_enum      ("Port 1 device",     0, in_type_sel1,    men_in_type_sel),
+	mee_enum      ("Port 2 device",     0, in_type_sel2,    men_in_type_sel),
+	mee_onoff_h   ("Nubs as buttons",   MA_CTRL_NUBS_BTNS,  in_evdev_allow_abs_only, 1, h_nub_btns),
+	mee_range     ("Analog deadzone",   MA_CTRL_DEADZONE,   analog_deadzone, 1, 99),
+	mee_onoff_h   ("No TS Gun trigger", 0, g_opts, OPT_TSGUN_NOTRIGGER, h_notsgun),
 	mee_cust_nosave("Save global config",       MA_OPT_SAVECFG,      mh_savecfg, mgn_saveloadcfg),
 	mee_cust_nosave("Save cfg for loaded game", MA_OPT_SAVECFG_GAME, mh_savecfg, mgn_saveloadcfg),
 	mee_handler   ("Rescan devices",  mh_input_rescan),
@@ -960,13 +1013,13 @@ static int menu_loop_keyconfig(int id, int keys)
 	static int sel = 0;
 
 //	me_enable(e_menu_keyconfig, MA_OPT_SAVECFG_GAME, ready_to_go && CdromId[0]);
-	me_loop(e_menu_keyconfig, &sel, NULL);
+	me_loop(e_menu_keyconfig, &sel);
 	return 0;
 }
 
 // ------------ gfx options menu ------------
 
-static const char *men_scaler[] = { "1x1", "scaled 4:3", "fullscreen", "custom", NULL };
+static const char *men_scaler[] = { "1x1", "scaled 4:3", "integer scaled 4:3", "fullscreen", "custom", NULL };
 static const char h_cscaler[]   = "Displays the scaler layer, you can resize it\n"
 				  "using d-pad or move it using R+d-pad";
 static const char *men_dummy[] = { NULL };
@@ -1034,13 +1087,28 @@ static int menu_loop_gfx_options(int id, int keys)
 {
 	static int sel = 0;
 
-	me_loop(e_menu_gfx_options, &sel, NULL);
+	me_loop(e_menu_gfx_options, &sel);
 
 	return 0;
 }
 
 // ------------ bios/plugins ------------
 
+static menu_entry e_menu_plugin_gpu_unai[] =
+{
+	mee_onoff     ("Abe's Odyssey hack",         0, pl_rearmed_cbs.gpu_unai.abe_hack, 1),
+	mee_onoff     ("Disable lighting",           0, pl_rearmed_cbs.gpu_unai.no_light, 1),
+	mee_onoff     ("Disable blending",           0, pl_rearmed_cbs.gpu_unai.no_blend, 1),
+	mee_end,
+};
+
+static int menu_loop_plugin_gpu_unai(int id, int keys)
+{
+	int sel = 0;
+	me_loop(e_menu_plugin_gpu_unai, &sel);
+	return 0;
+}
+
 static const char *men_gpu_dithering[] = { "None", "Game dependant", "Always", NULL };
 static const char h_gpu_0[]            = "Needed for Chrono Cross";
 static const char h_gpu_1[]            = "Capcom fighting games";
@@ -1052,36 +1120,37 @@ static const char h_gpu_8[]            = "Needed by Dark Forces";
 static const char h_gpu_9[]            = "better g-colors, worse textures";
 static const char h_gpu_10[]           = "Toggle busy flags after drawing";
 
-static menu_entry e_menu_plugin_gpu[] =
+static menu_entry e_menu_plugin_gpu_peops[] =
 {
-	mee_enum      ("Dithering",                  0, iUseDither, men_gpu_dithering),
-	mee_onoff_h   ("Odd/even bit hack",          0, dwActFixes, 1<<0, h_gpu_0),
-	mee_onoff_h   ("Expand screen width",        0, dwActFixes, 1<<1, h_gpu_1),
-	mee_onoff_h   ("Ignore brightness color",    0, dwActFixes, 1<<2, h_gpu_2),
-	mee_onoff_h   ("Disable coordinate check",   0, dwActFixes, 1<<3, h_gpu_3),
-	mee_onoff_h   ("Lazy screen update",         0, dwActFixes, 1<<6, h_gpu_6),
-	mee_onoff_h   ("Old frame skipping",         0, dwActFixes, 1<<7, h_gpu_7),
-	mee_onoff_h   ("Repeated flat tex triangles ",0,dwActFixes, 1<<8, h_gpu_8),
-	mee_onoff_h   ("Draw quads with triangles",  0, dwActFixes, 1<<9, h_gpu_9),
-	mee_onoff_h   ("Fake 'gpu busy' states",     0, dwActFixes, 1<<10, h_gpu_10),
+	mee_enum      ("Dithering",                  0, pl_rearmed_cbs.gpu_peops.iUseDither, men_gpu_dithering),
+	mee_onoff_h   ("Odd/even bit hack",          0, pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<0, h_gpu_0),
+	mee_onoff_h   ("Expand screen width",        0, pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<1, h_gpu_1),
+	mee_onoff_h   ("Ignore brightness color",    0, pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<2, h_gpu_2),
+	mee_onoff_h   ("Disable coordinate check",   0, pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<3, h_gpu_3),
+	mee_onoff_h   ("Lazy screen update",         0, pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<6, h_gpu_6),
+	mee_onoff_h   ("Old frame skipping",         0, pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<7, h_gpu_7),
+	mee_onoff_h   ("Repeated flat tex triangles ",0,pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<8, h_gpu_8),
+	mee_onoff_h   ("Draw quads with triangles",  0, pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<9, h_gpu_9),
+	mee_onoff_h   ("Fake 'gpu busy' states",     0, pl_rearmed_cbs.gpu_peops.dwActFixes, 1<<10, h_gpu_10),
 	mee_end,
 };
 
-static int menu_loop_plugin_gpu(int id, int keys)
+static int menu_loop_plugin_gpu_peops(int id, int keys)
 {
 	static int sel = 0;
-	me_loop(e_menu_plugin_gpu, &sel, NULL);
+	me_loop(e_menu_plugin_gpu_peops, &sel);
 	return 0;
 }
 
-static const char *men_spu_reverb[] = { "Off", "Fake", "On", NULL };
 static const char *men_spu_interp[] = { "None", "Simple", "Gaussian", "Cubic", NULL };
+static const char h_spu_volboost[]  = "Large values cause distortion";
 static const char h_spu_irq_wait[]  = "Wait for CPU (recommended set to ON)";
 static const char h_spu_thread[]    = "Run sound emulation in main thread (recommended)";
 
 static menu_entry e_menu_plugin_spu[] =
 {
-	mee_enum      ("Reverb",                    0, iUseReverb, men_spu_reverb),
+	mee_range_h   ("Volume boost",              0, volume_boost, -5, 30, h_spu_volboost),
+	mee_onoff     ("Reverb",                    0, iUseReverb, 2),
 	mee_enum      ("Interpolation",             0, iUseInterpolation, men_spu_interp),
 	mee_onoff     ("Adjust XA pitch",           0, iXAPitch, 1),
 	mee_onoff_h   ("SPU IRQ Wait",              0, iSPUIRQWait, 1, h_spu_irq_wait),
@@ -1092,16 +1161,17 @@ static menu_entry e_menu_plugin_spu[] =
 static int menu_loop_plugin_spu(int id, int keys)
 {
 	static int sel = 0;
-	me_loop(e_menu_plugin_spu, &sel, NULL);
+	me_loop(e_menu_plugin_spu, &sel);
 	return 0;
 }
 
-static const char h_bios[]       = "HLE is simulated BIOS. BIOS selection is saved in savestates\n"
-				   "and can't be changed there. Must save config and reload\n"
-				   "the game for change to take effect";
+static const char h_bios[]       = "HLE is simulated BIOS. BIOS selection is saved in\n"
+				   "savestates and can't be changed there. Must save\n"
+				   "config and reload the game for change to take effect";
 static const char h_plugin_xpu[] = "Must save config and reload the game\n"
 				   "for plugin change to take effect";
-static const char h_gpu[]        = "Configure built-in P.E.Op.S. SoftGL Driver V1.17";
+static const char h_gpu_peops[]  = "Configure P.E.Op.S. SoftGL Driver V1.17";
+static const char h_gpu_unai[]   = "Configure Unai/PCSX4ALL Team GPU plugin";
 static const char h_spu[]        = "Configure built-in P.E.Op.S. Sound Driver V1.7";
 
 static menu_entry e_menu_plugin_options[] =
@@ -1109,44 +1179,55 @@ static menu_entry e_menu_plugin_options[] =
 	mee_enum_h    ("BIOS",                          0, bios_sel, bioses, h_bios),
 	mee_enum_h    ("GPU plugin",                    0, gpu_plugsel, gpu_plugins, h_plugin_xpu),
 	mee_enum_h    ("SPU plugin",                    0, spu_plugsel, spu_plugins, h_plugin_xpu),
-	mee_handler_h ("Configure built-in GPU plugin", menu_loop_plugin_gpu, h_gpu),
+	mee_handler_h ("Configure gpu_peops plugin",    menu_loop_plugin_gpu_peops, h_gpu_peops),
+	mee_handler_h ("Configure PCSX4ALL GPU plugin", menu_loop_plugin_gpu_unai, h_gpu_unai),
 	mee_handler_h ("Configure built-in SPU plugin", menu_loop_plugin_spu, h_spu),
 	mee_end,
 };
 
-static menu_entry e_menu_main[];
+static menu_entry e_menu_main2[];
 
 static int menu_loop_plugin_options(int id, int keys)
 {
 	static int sel = 0;
-	me_loop(e_menu_plugin_options, &sel, NULL);
+	me_loop(e_menu_plugin_options, &sel);
 
 	// sync BIOS/plugins
 	snprintf(Config.Bios, sizeof(Config.Bios), "%s", bioses[bios_sel]);
 	snprintf(Config.Gpu, sizeof(Config.Gpu), "%s", gpu_plugins[gpu_plugsel]);
 	snprintf(Config.Spu, sizeof(Config.Spu), "%s", spu_plugins[spu_plugsel]);
-	me_enable(e_menu_main, MA_MAIN_RUN_BIOS, bios_sel != 0);
+	me_enable(e_menu_main2, MA_MAIN_RUN_BIOS, bios_sel != 0);
 
 	return 0;
 }
 
 // ------------ adv options menu ------------
 
+static const char *men_cfg_cdrr[] = { "Auto", "ON", "OFF", NULL };
 static const char h_cfg_cpul[]   = "Shows CPU usage in %";
+static const char h_cfg_spu[]    = "Shows active SPU channels\n"
+				   "(green: normal, red: fmod, blue: noise)";
 static const char h_cfg_fl[]     = "Frame Limiter keeps the game from running too fast";
 static const char h_cfg_xa[]     = "Disables XA sound, which can sometimes improve performance";
 static const char h_cfg_cdda[]   = "Disable CD Audio for a performance boost\n"
 				   "(proper .cue/.bin dump is needed otherwise)";
-static const char h_cfg_sio[]    = "This should be enabled for certain memcards/gamepads";
-static const char h_cfg_spuirq[] = "Compatibility tweak; should probably be left off";
-static const char h_cfg_rcnt1[]  = "Parasite Eve 2, Vandal Hearts 1/2 Fix";
-static const char h_cfg_rcnt2[]  = "InuYasha Sengoku Battle Fix";
+static const char h_cfg_sio[]    = "You should not need this, breaks games";
+static const char h_cfg_spuirq[] = "Compatibility tweak; should be left off";
+static const char h_cfg_rcnt1[]  = "Parasite Eve 2, Vandal Hearts 1/2 Fix\n"
+				   "(timing hack, breaks other games)";
+static const char h_cfg_rcnt2[]  = "InuYasha Sengoku Battle Fix\n"
+				   "(timing hack, breaks other games)";
+static const char h_cfg_cdrr[]   = "Compatibility tweak (fixes Team Buddies, maybe more)\n"
+				   "(CD timing hack, breaks FMVs)";
+static const char h_cfg_psxclk[] = "Over/under-clock the PSX, default is " DEFAULT_PSX_CLOCK_S "\n"
+				   "(may break games, must reload game to take effect)";
 static const char h_cfg_nodrc[]  = "Disable dynamic recompiler and use interpreter\n"
 				   "Might be useful to overcome some dynarec bugs";
 
 static menu_entry e_menu_adv_options[] =
 {
 	mee_onoff_h   ("Show CPU load",          0, g_opts, OPT_SHOWCPU, h_cfg_cpul),
+	mee_onoff_h   ("Show SPU channels",      0, g_opts, OPT_SHOWSPU, h_cfg_spu),
 	mee_onoff_h   ("Disable Frame Limiter",  0, g_opts, OPT_NO_FRAMELIM, h_cfg_fl),
 	mee_onoff_h   ("Disable XA Decoding",    0, Config.Xa, 1, h_cfg_xa),
 	mee_onoff_h   ("Disable CD Audio",       0, Config.Cdda, 1, h_cfg_cdda),
@@ -1154,6 +1235,8 @@ static menu_entry e_menu_adv_options[] =
 	mee_onoff_h   ("SPU IRQ Always Enabled", 0, Config.SpuIrq, 1, h_cfg_spuirq),
 	mee_onoff_h   ("Rootcounter hack",       0, Config.RCntFix, 1, h_cfg_rcnt1),
 	mee_onoff_h   ("Rootcounter hack 2",     0, Config.VSyncWA, 1, h_cfg_rcnt2),
+	mee_enum_h    ("CD read reschedule hack",0, Config.CdrReschedule, men_cfg_cdrr, h_cfg_cdrr),
+	mee_range_h   ("PSX CPU clock, %%",      0, psx_clock, 1, 500, h_cfg_psxclk),
 	mee_onoff_h   ("Disable dynarec (slow!)",0, Config.Cpu, 1, h_cfg_nodrc),
 	mee_end,
 };
@@ -1161,7 +1244,7 @@ static menu_entry e_menu_adv_options[] =
 static int menu_loop_adv_options(int id, int keys)
 {
 	static int sel = 0;
-	me_loop(e_menu_adv_options, &sel, NULL);
+	me_loop(e_menu_adv_options, &sel);
 	return 0;
 }
 
@@ -1175,6 +1258,7 @@ static int mh_restore_defaults(int id, int keys)
 }
 
 static const char *men_region[]       = { "Auto", "NTSC", "PAL", NULL };
+static const char *men_frameskip[]    = { "Auto", "Off", "1", NULL };
 /*
 static const char *men_confirm_save[] = { "OFF", "writes", "loads", "both", NULL };
 static const char h_confirm_save[]    = "Ask for confirmation when overwriting save,\n"
@@ -1188,11 +1272,11 @@ static menu_entry e_menu_options[] =
 {
 //	mee_range     ("Save slot",                0, state_slot, 0, 9),
 //	mee_enum_h    ("Confirm savestate",        0, dummy, men_confirm_save, h_confirm_save),
-	mee_onoff_h   ("Frameskip",                0, UseFrameSkip, 1, h_frameskip),
+	mee_enum_h    ("Frameskip",                0, frameskip, men_frameskip, h_frameskip),
 	mee_onoff     ("Show FPS",                 0, g_opts, OPT_SHOWFPS),
 	mee_enum      ("Region",                   0, region, men_region),
 	mee_range     ("CPU clock",                MA_OPT_CPU_CLOCKS, cpu_clock, 20, 5000),
-	mee_handler   ("[Display]",                menu_loop_gfx_options),
+	mee_handler_id("[Display]",                MA_OPT_DISP_OPTS, menu_loop_gfx_options),
 	mee_handler   ("[BIOS/Plugins]",           menu_loop_plugin_options),
 	mee_handler   ("[Advanced]",               menu_loop_adv_options),
 	mee_cust_nosave("Save global config",      MA_OPT_SAVECFG,      mh_savecfg, mgn_saveloadcfg),
@@ -1210,7 +1294,7 @@ static int menu_loop_options(int id, int keys)
 	e_menu_options[i].enabled = cpu_clock != 0 ? 1 : 0;
 	me_enable(e_menu_options, MA_OPT_SAVECFG_GAME, ready_to_go && CdromId[0]);
 
-	me_loop(e_menu_options, &sel, NULL);
+	me_loop(e_menu_options, &sel);
 
 	return 0;
 }
@@ -1264,20 +1348,136 @@ static void debug_menu_loop(void)
 	free(gpuf);
 }
 
-// ------------ main menu ------------
+// --------- memcard manager ---------
+
+static void draw_mc_icon(int dx, int dy, const u16 *s)
+{
+	u16 *d;
+	int x, y, l, p;
+	
+	d = (u16 *)g_menuscreen_ptr + g_menuscreen_w * dy + dx;
+
+	for (y = 0; y < 16; y++, s += 16) {
+		for (l = 0; l < 2; l++, d += g_menuscreen_w) {
+			for (x = 0; x < 16; x++) {
+				p = s[x];
+				d[x*2] = d[x*2 + 1] = ((p & 0x7c00) >> 10)
+					| ((p & 0x03e0) << 1) | ((p & 0x1f) << 11);
+			}
+		}
+	}
+}
+
+static void draw_mc_bg(void)
+{
+	McdBlock *blocks1, *blocks2;
+	int maxicons = 15;
+	int i, y, row2;
+
+	blocks1 = malloc(15 * sizeof(blocks1[0]));
+	blocks2 = malloc(15 * sizeof(blocks1[0]));
+	if (blocks1 == NULL || blocks2 == NULL)
+		goto out;
+
+	for (i = 0; i < 15; i++) {
+		GetMcdBlockInfo(1, i + 1, &blocks1[i]);
+		GetMcdBlockInfo(2, i + 1, &blocks2[i]);
+	}
+
+	menu_draw_begin(1);
+
+	memcpy(g_menuscreen_ptr, g_menubg_src_ptr, g_menuscreen_w * g_menuscreen_h * 2);
+
+	y = g_menuscreen_h / 2 - 15 * 32 / 2;
+	if (y < 0) {
+		// doesn't fit..
+		y = 0;
+		maxicons = g_menuscreen_h / 32;
+	}
+
+	row2 = g_menuscreen_w / 2;
+	for (i = 0; i < maxicons; i++) {
+		draw_mc_icon(8, y + i * 32, (u16 *)blocks1[i].Icon);
+		smalltext_out16(10+32, y + i * 32 + 8, blocks1[i].sTitle, 0xf71e);
+
+		draw_mc_icon(row2 + 8, y + i * 32, (u16 *)blocks2[i].Icon);
+		smalltext_out16(row2 + 10+32, y + i * 32 + 8, blocks2[i].sTitle, 0xf71e);
+	}
+
+	menu_darken_bg(g_menubg_ptr, g_menuscreen_ptr, g_menuscreen_w * g_menuscreen_h, 0);
+
+	menu_draw_end();
+out:
+	free(blocks1);
+	free(blocks2);
+}
+
+static void handle_memcard_sel(void)
+{
+	Config.Mcd1[0] = 0;
+	if (memcard1_sel != 0)
+		snprintf(Config.Mcd1, sizeof(Config.Mcd1), ".%s%s", MEMCARD_DIR, memcards[memcard1_sel]);
+	Config.Mcd2[0] = 0;
+	if (memcard2_sel != 0)
+		snprintf(Config.Mcd2, sizeof(Config.Mcd2), ".%s%s", MEMCARD_DIR, memcards[memcard2_sel]);
+	LoadMcds(Config.Mcd1, Config.Mcd2);
+	draw_mc_bg();
+}
+
+static menu_entry e_memcard_options[] =
+{
+	mee_enum("Memory card 1", 0, memcard1_sel, memcards),
+	mee_enum("Memory card 2", 0, memcard2_sel, memcards),
+	mee_end,
+};
+
+static int menu_loop_memcards(int id, int keys)
+{
+	static int sel = 0;
+	char *p;
+	int i;
+
+	memcard1_sel = memcard2_sel = 0;
+	p = strrchr(Config.Mcd1, '/');
+	if (p != NULL)
+		for (i = 0; memcards[i] != NULL; i++)
+			if (strcmp(p + 1, memcards[i]) == 0)
+				{ memcard1_sel = i; break; }
+	p = strrchr(Config.Mcd2, '/');
+	if (p != NULL)
+		for (i = 0; memcards[i] != NULL; i++)
+			if (strcmp(p + 1, memcards[i]) == 0)
+				{ memcard2_sel = i; break; }
+
+	me_loop_d(e_memcard_options, &sel, handle_memcard_sel, NULL);
+
+	memcpy(g_menubg_ptr, g_menubg_src_ptr, g_menuscreen_w * g_menuscreen_h * 2);
+
+	return 0;
+}
+
+// --------- main menu help ----------
 
 static void menu_bios_warn(void)
 {
 	int inp;
 	static const char msg[] =
-		"You don't seem to have copied any BIOS files to\n"
+		"You don't seem to have copied any BIOS\n"
+		"files to\n"
+#ifdef __ARM_ARCH_7A__ // XXX
 		"<SD card>/pandora/appdata/pcsx_rearmed/bios/\n\n"
-		"While many games work fine with fake (HLE) BIOS,\n"
-		"others (like MGS and FF8) require BIOS to work.\n"
-		"After copying the file, you'll also need to\n"
-		"select it in the emu's options->[BIOS/Plugins]\n\n"
-		"The file is usually named SCPH1001.BIN, but\n"
-		"other not compressed files can be used too.\n\n"
+#else
+		"pcsx_rearmed/bios/\n\n"
+#endif
+		"While many games work fine with fake\n"
+		"(HLE) BIOS, others (like MGS and FF8)\n"
+		"require BIOS to work.\n"
+		"After copying the file, you'll also need\n"
+		"to select it in the emu's menu:\n"
+		"options->[BIOS/Plugins]\n\n"
+		"The file is usually named SCPH1001.BIN,\n"
+		"but other not compressed files can be\n"
+		"used too.\n\n"
 		"Press (B) or (X) to continue";
 
 	while (1)
@@ -1296,13 +1496,25 @@ void OnFile_Exit();
 
 static void draw_frame_main(void)
 {
+	struct tm *tmp;
+	time_t ltime;
+	char ltime_s[16];
+	char buff[64];
+
 	if (CdromId[0] != 0) {
-		char buff[64];
 		snprintf(buff, sizeof(buff), "%.32s/%.9s (running as %s, with %s)",
 			 get_cd_label(), CdromId, Config.PsxType ? "PAL" : "NTSC",
 			 Config.HLE ? "HLE" : "BIOS");
 		smalltext_out16(4, 1, buff, 0x105f);
 	}
+
+	if (ready_to_go) {
+		ltime = time(NULL);
+		tmp = localtime(&ltime);
+		strftime(ltime_s, sizeof(ltime_s), "%H:%M", tmp);
+		snprintf(buff, sizeof(buff), "%s %3d%%", ltime_s, plat_get_bat_capacity());
+		smalltext_out16(4, 1 + me_sfont_h, buff, 0x105f);
+	}
 }
 
 static void draw_frame_credits(void)
@@ -1338,16 +1550,13 @@ static int reset_game(void)
 	return 0;
 }
 
-static int run_bios(void)
+static int reload_plugins(const char *cdimg)
 {
-	if (bios_sel == 0)
-		return -1;
-
-	ready_to_go = 0;
-	pl_fbdev_buf = NULL;
+	pl_vout_buf = NULL;
 
 	ClosePlugins();
-	set_cd_image(NULL);
+
+	set_cd_image(cdimg);
 	LoadPlugins();
 	pcnt_hook_plugins();
 	NetOpened = 0;
@@ -1360,27 +1569,53 @@ static int run_bios(void)
 	CdromId[0] = '\0';
 	CdromLabel[0] = '\0';
 
+	return 0;
+}
+
+static int run_bios(void)
+{
+	if (bios_sel == 0)
+		return -1;
+
+	ready_to_go = 0;
+	if (reload_plugins(NULL) != 0)
+		return -1;
 	SysReset();
 
 	ready_to_go = 1;
 	return 0;
 }
 
+static int run_exe(void)
+{
+	const char *fname;
+
+	fname = menu_loop_romsel(last_selected_fname, sizeof(last_selected_fname));
+	if (fname == NULL)
+		return -1;
+
+	ready_to_go = 0;
+	if (reload_plugins(NULL) != 0)
+		return -1;
+
+	SysReset();
+	if (Load(fname) != 0) {
+		me_update_msg("exe load failed, bad file?");
+		printf("meh\n");
+		return -1;
+	}
+
+	ready_to_go = 1;
+	return 0;
+}
+
 static int run_cd_image(const char *fname)
 {
 	ready_to_go = 0;
-	pl_fbdev_buf = NULL;
+	reload_plugins(fname);
 
-	ClosePlugins();
-	set_cd_image(fname);
-	LoadPlugins();
-	pcnt_hook_plugins();
-	NetOpened = 0;
-	if (OpenPlugins() == -1) {
-		me_update_msg("failed to open plugins");
-		return -1;
-	}
-	plugin_call_rearmed_cbs();
+	// always autodetect, menu_sync_config will override as needed
+	Config.PsxAuto = 1;
 
 	if (CheckCdrom() == -1) {
 		// Only check the CD if we are starting the console with a CD
@@ -1405,7 +1640,7 @@ static int run_cd_image(const char *fname)
 static int romsel_run(void)
 {
 	int prev_gpu, prev_spu;
-	char *fname;
+	const char *fname;
 
 	fname = menu_loop_romsel(last_selected_fname, sizeof(last_selected_fname));
 	if (fname == NULL)
@@ -1497,6 +1732,10 @@ static int main_menu_handler(int id, int keys)
 		if (run_bios() == 0)
 			return 1;
 		break;
+	case MA_MAIN_RUN_EXE:
+		if (run_exe() == 0)
+			return 1;
+		break;
 	case MA_MAIN_CREDITS:
 		draw_menu_message(credits_text, draw_frame_credits);
 		in_menu_wait(PBTN_MOK|PBTN_MBACK, 70);
@@ -1512,6 +1751,27 @@ static int main_menu_handler(int id, int keys)
 	return 0;
 }
 
+static menu_entry e_menu_main2[] =
+{
+	mee_handler_id("Change CD image",    MA_MAIN_SWAP_CD,     main_menu_handler),
+	mee_handler_id("Run BIOS",           MA_MAIN_RUN_BIOS,    main_menu_handler),
+	mee_handler_id("Run EXE",            MA_MAIN_RUN_EXE,     main_menu_handler),
+	mee_handler   ("Memcard manager",    menu_loop_memcards),
+	mee_end,
+};
+
+static int main_menu2_handler(int id, int keys)
+{
+	static int sel = 0;
+
+	me_enable(e_menu_main2, MA_MAIN_SWAP_CD,  ready_to_go);
+	me_enable(e_menu_main2, MA_MAIN_RUN_BIOS, bios_sel != 0);
+
+	return me_loop_d(e_menu_main2, &sel, NULL, draw_frame_main);
+}
+
+static const char h_extra[] = "Change CD, manage memcards..\n";
+
 static menu_entry e_menu_main[] =
 {
 	mee_label     (""),
@@ -1521,10 +1781,9 @@ static menu_entry e_menu_main[] =
 	mee_handler_id("Load State",         MA_MAIN_LOAD_STATE,  main_menu_handler),
 	mee_handler_id("Reset game",         MA_MAIN_RESET_GAME,  main_menu_handler),
 	mee_handler_id("Load CD image",      MA_MAIN_LOAD_ROM,    main_menu_handler),
-	mee_handler_id("Change CD image",    MA_MAIN_SWAP_CD,     main_menu_handler),
-	mee_handler_id("Run BIOS",           MA_MAIN_RUN_BIOS,    main_menu_handler),
 	mee_handler   ("Options",            menu_loop_options),
 	mee_handler   ("Controls",           menu_loop_keyconfig),
+	mee_handler_h ("Extra stuff",        main_menu2_handler,  h_extra),
 	mee_handler_id("Credits",            MA_MAIN_CREDITS,     main_menu_handler),
 	mee_handler_id("Exit",               MA_MAIN_EXIT,        main_menu_handler),
 	mee_end,
@@ -1549,13 +1808,11 @@ void menu_loop(void)
 	me_enable(e_menu_main, MA_MAIN_SAVE_STATE,  ready_to_go && CdromId[0]);
 	me_enable(e_menu_main, MA_MAIN_LOAD_STATE,  ready_to_go && CdromId[0]);
 	me_enable(e_menu_main, MA_MAIN_RESET_GAME,  ready_to_go);
-	me_enable(e_menu_main, MA_MAIN_SWAP_CD,  ready_to_go);
-	me_enable(e_menu_main, MA_MAIN_RUN_BIOS, bios_sel != 0);
 
 	in_set_config_int(0, IN_CFG_BLOCKING, 1);
 
 	do {
-		me_loop(e_menu_main, &sel, draw_frame_main);
+		me_loop_d(e_menu_main, &sel, NULL, draw_frame_main);
 	} while (!ready_to_go);
 
 	/* wait until menu, ok, back is released */
@@ -1567,18 +1824,26 @@ void menu_loop(void)
 	menu_prepare_emu();
 }
 
+static int qsort_strcmp(const void *p1, const void *p2)
+{
+	char * const *s1 = (char * const *)p1;
+	char * const *s2 = (char * const *)p2;
+	return strcasecmp(*s1, *s2);
+}
+
 static void scan_bios_plugins(void)
 {
 	char fname[MAXPATHLEN];
 	struct dirent *ent;
-	int bios_i, gpu_i, spu_i;
+	int bios_i, gpu_i, spu_i, mc_i;
 	char *p;
 	DIR *dir;
 
 	bioses[0] = "HLE";
 	gpu_plugins[0] = "builtin_gpu";
 	spu_plugins[0] = "builtin_spu";
-	bios_i = gpu_i = spu_i = 1;
+	memcards[0] = "(none)";
+	bios_i = gpu_i = spu_i = mc_i = 1;
 
 	snprintf(fname, sizeof(fname), "%s/", Config.BiosDir);
 	dir = opendir(fname);
@@ -1621,8 +1886,8 @@ do_plugins:
 	snprintf(fname, sizeof(fname), "%s/", Config.PluginsDir);
 	dir = opendir(fname);
 	if (dir == NULL) {
-		perror("scan_bios_plugins opendir");
-		return;
+		perror("scan_bios_plugins plugins opendir");
+		goto do_memcards;
 	}
 
 	while (1) {
@@ -1668,6 +1933,46 @@ do_plugins:
 	}
 
 	closedir(dir);
+
+do_memcards:
+	dir = opendir("." MEMCARD_DIR);
+	if (dir == NULL) {
+		perror("scan_bios_plugins memcards opendir");
+		return;
+	}
+
+	while (1) {
+		struct stat st;
+
+		errno = 0;
+		ent = readdir(dir);
+		if (ent == NULL) {
+			if (errno != 0)
+				perror("readdir");
+			break;
+		}
+
+		if (ent->d_type != DT_REG && ent->d_type != DT_LNK)
+			continue;
+
+		snprintf(fname, sizeof(fname), "." MEMCARD_DIR "%s", ent->d_name);
+		if (stat(fname, &st) != 0) {
+			printf("bad memcard file: %s\n", ent->d_name);
+			continue;
+		}
+
+		if (mc_i < ARRAY_SIZE(memcards) - 1) {
+			memcards[mc_i++] = strdup(ent->d_name);
+			continue;
+		}
+
+		printf("too many memcards, dropping \"%s\"\n", ent->d_name);
+	}
+
+	if (mc_i > 2)
+		qsort(memcards + 1, mc_i - 1, sizeof(memcards[0]), qsort_strcmp);
+
+	closedir(dir);
 }
 
 void menu_init(void)
@@ -1691,18 +1996,68 @@ void menu_init(void)
 		exit(1);
 	emu_make_path(buff, "skin/background.png", sizeof(buff));
 	readpng(g_menubg_src_ptr, buff, READPNG_BG, g_menuscreen_w, g_menuscreen_h);
+
+#ifndef __ARM_ARCH_7A__ /* XXX */
+	me_enable(e_menu_options, MA_OPT_DISP_OPTS, 0);
+	me_enable(e_menu_keyconfig, MA_CTRL_NUBS_BTNS, 0);
+#else
+	me_enable(e_menu_keyconfig, MA_CTRL_DEADZONE, 0);
+#endif
 }
 
 void menu_notify_mode_change(int w, int h, int bpp)
 {
+	float mult;
+	int imult;
+
 	last_psx_w = w;
 	last_psx_h = h;
 	last_psx_bpp = bpp;
 
-	if (scaling == SCALE_1_1) {
-		g_layer_x = 800/2 - w/2;  g_layer_y = 480/2 - h/2;
+	switch (scaling) {
+	case SCALE_1_1:
 		g_layer_w = w; g_layer_h = h;
+		break;
+
+	case SCALE_4_3v2:
+		if (h > g_menuscreen_h || (240 < h && h <= 360))
+			goto fractional_4_3;
+
+		// 4:3 that prefers integer scaling
+		imult = g_menuscreen_h / h;
+		g_layer_w = w * imult;
+		g_layer_h = h * imult;
+		mult = (float)g_layer_w / (float)g_layer_h;
+		if (mult < 1.25f || mult > 1.666f)
+			g_layer_w = 4.0f/3.0f * (float)g_layer_h;
+		printf("  -> %dx%d %.1f\n", g_layer_w, g_layer_h, mult);
+		break;
+
+	fractional_4_3:
+	case SCALE_4_3:
+		mult = 240.0f / (float)h * 4.0f / 3.0f;
+		if (h > 256)
+			mult *= 2.0f;
+		g_layer_w = mult * (float)g_menuscreen_h;
+		g_layer_h = g_menuscreen_h;
+		printf("  -> %dx%d %.1f\n", g_layer_w, g_layer_h, mult);
+		break;
+
+	case SCALE_FULLSCREEN:
+		g_layer_w = g_menuscreen_w;
+		g_layer_h = g_menuscreen_h;
+		break;
+
+	default:
+		break;
 	}
+
+	g_layer_x = g_menuscreen_w / 2 - g_layer_w / 2;
+	g_layer_y = g_menuscreen_h / 2 - g_layer_h / 2;
+	if (g_layer_x < 0) g_layer_x = 0;
+	if (g_layer_y < 0) g_layer_y = 0;
+	if (g_layer_w > g_menuscreen_w) g_layer_w = g_menuscreen_w;
+	if (g_layer_h > g_menuscreen_h) g_layer_w = g_menuscreen_h;
 }
 
 static void menu_leave_emu(void)
@@ -1713,23 +2068,23 @@ static void menu_leave_emu(void)
 			fprintf(stderr, "Warning: GPU_close returned %d\n", ret);
 	}
 
+	plat_video_menu_enter(ready_to_go);
+
 	memcpy(g_menubg_ptr, g_menubg_src_ptr, g_menuscreen_w * g_menuscreen_h * 2);
-	if (pl_fbdev_buf != NULL && ready_to_go && last_psx_bpp == 16) {
+	if (pl_vout_buf != NULL && ready_to_go && last_psx_bpp == 16) {
 		int x = max(0, g_menuscreen_w - last_psx_w);
 		int y = max(0, g_menuscreen_h / 2 - last_psx_h / 2);
 		int w = min(g_menuscreen_w, last_psx_w);
 		int h = min(g_menuscreen_h, last_psx_h);
 		u16 *d = (u16 *)g_menubg_ptr + g_menuscreen_w * y + x;
-		u16 *s = pl_fbdev_buf;
+		u16 *s = pl_vout_buf;
 
 		for (; h > 0; h--, d += g_menuscreen_w, s += last_psx_w)
 			menu_darken_bg(d, s, w, 0);
 	}
 
 	if (ready_to_go)
-		cpu_clock = get_cpu_clock();
-
-	plat_video_menu_enter(ready_to_go);
+		cpu_clock = plat_cpu_clock_get();
 }
 
 void menu_prepare_emu(void)
@@ -1738,21 +2093,7 @@ void menu_prepare_emu(void)
 
 	plat_video_menu_leave();
 
-	switch (scaling) {
-	case SCALE_1_1:
-		menu_notify_mode_change(last_psx_w, last_psx_h, last_psx_bpp);
-		break;
-	case SCALE_4_3:
-		g_layer_x = 80;  g_layer_y = 0;
-		g_layer_w = 640; g_layer_h = 480;
-		break;
-	case SCALE_FULLSCREEN:
-		g_layer_x = 0;   g_layer_y = 0;
-		g_layer_w = 800; g_layer_h = 480;
-		break;
-	case SCALE_CUSTOM:
-		break;
-	}
+	menu_notify_mode_change(last_psx_w, last_psx_h, last_psx_bpp);
 
 	psxCpu = (Config.Cpu == CPU_INTERPRETER) ? &psxInt : &psxRec;
 	if (psxCpu != prev_cpu)
@@ -1767,7 +2108,10 @@ void menu_prepare_emu(void)
 	menu_sync_config();
 	apply_lcdrate(Config.PsxType);
 	apply_filter(filter);
-	apply_cpu_clock();
+	plat_cpu_clock_apply(cpu_clock);
+
+	// push config to GPU plugin
+	plugin_call_rearmed_cbs();
 
 	if (GPU_open != NULL) {
 		int ret = GPU_open(&gpuDisp, "PCSX", NULL);
@@ -1775,7 +2119,7 @@ void menu_prepare_emu(void)
 			fprintf(stderr, "Warning: GPU_open returned %d\n", ret);
 	}
 
-	dfinput_activate(in_type == PSE_PAD_TYPE_ANALOGPAD);
+	dfinput_activate();
 }
 
 void me_update_msg(const char *msg)
