@@ -31,6 +31,7 @@
 #include <pthread.h>
 #include <sys/time.h>
 #endif
+#include <zlib.h>
 
 static FILE *cdHandle = NULL;
 static FILE *cddaHandle = NULL;
@@ -38,6 +39,7 @@ static FILE *subHandle = NULL;
 
 static boolean subChanMixed = FALSE;
 static boolean subChanRaw = FALSE;
+static boolean subChanMissing = FALSE;
 
 static unsigned char cdbuffer[DATA_SIZE];
 static unsigned char subbuffer[SUB_FRAMESIZE];
@@ -54,8 +56,25 @@ static pthread_t threadid;
 static unsigned int initial_offset = 0;
 static boolean playing = FALSE;
 static boolean cddaBigEndian = FALSE;
-static unsigned int cddaCurOffset = 0;
-static unsigned int cddaStartOffset;
+static unsigned int cdda_cur_sector;
+static unsigned int cdda_start_sector;
+/* Frame offset into CD image where pregap data would be found if it was there.
+ * If a game seeks there we must *not* return subchannel data since it's
+ * not in the CD image, so that cdrom code can fake subchannel data instead.
+ * XXX: there could be multiple pregaps but PSX dumps only have one? */
+static unsigned int pregapOffset;
+
+// compressed image stuff
+static struct {
+	unsigned char buff_raw[16][CD_FRAMESIZE_RAW];
+	unsigned char buff_compressed[CD_FRAMESIZE_RAW * 16 + 100];
+	unsigned int *index_table;
+	unsigned int index_len;
+	unsigned int current_block;
+	unsigned int sector_in_blk;
+} *compr_img;
+
+int (*cdimg_read_func)(FILE *f, void *dest, int sector, int offset);
 
 char* CALLBACK CDR__getDriveLetter(void);
 long CALLBACK CDR__configure(void);
@@ -64,6 +83,8 @@ void CALLBACK CDR__about(void);
 long CALLBACK CDR__setfilename(char *filename);
 long CALLBACK CDR__getStatus(struct CdrStat *stat);
 
+static void DecodeRawSubData(void);
+
 extern void *hCDRDriver;
 
 struct trackinfo {
@@ -71,6 +92,7 @@ struct trackinfo {
 	char start[3];		// MSF-format
 	char length[3];		// MSF-format
 	FILE *handle;		// for multi-track images CDDA
+	int start_offset;	// sector offset from start of above file
 };
 
 #define MAXTRACKS 100 /* How many tracks can a CD hold? */
@@ -142,51 +164,28 @@ static void playthread(void *param)
 static void *playthread(void *param)
 #endif
 {
-	long			d, t, i, s;
+	long osleep, d, t, i, s;
 	unsigned char	tmp;
+	int ret = 0;
 
 	t = GetTickCount();
 
 	while (playing) {
-		d = t - (long)GetTickCount();
-		if (d <= 0) {
-			d = 1;
-		}
-		else if (d > CDDA_FRAMETIME) {
-			d = CDDA_FRAMETIME;
-		}
-#ifdef _WIN32
-		Sleep(d);
-#else
-		usleep(d * 1000);
-#endif
-		// HACK: stop feeding data while emu is paused
-		extern int stop;
-		if (stop) {
-			usleep(100000);
-			continue;
+		s = 0;
+		for (i = 0; i < sizeof(sndbuffer) / CD_FRAMESIZE_RAW; i++) {
+			d = cdimg_read_func(cddaHandle, sndbuffer + s, cdda_cur_sector, 0);
+			if (d < CD_FRAMESIZE_RAW)
+				break;
+
+			s += d;
+			cdda_cur_sector++;
 		}
 
-		t = GetTickCount() + CDDA_FRAMETIME;
+		if (subHandle != NULL) {
+			fseek(subHandle, cdda_cur_sector * SUB_FRAMESIZE, SEEK_SET);
+			fread(subbuffer, 1, SUB_FRAMESIZE, subHandle);
 
-		if (subChanMixed) {
-			s = 0;
-
-			for (i = 0; i < sizeof(sndbuffer) / CD_FRAMESIZE_RAW; i++) {
-				// read one sector
-				d = fread(sndbuffer + CD_FRAMESIZE_RAW * i, 1, CD_FRAMESIZE_RAW, cddaHandle);
-				if (d < CD_FRAMESIZE_RAW) {
-					break;
-				}
-
-				s += d;
-
-				// skip the subchannel data
-				fseek(cddaHandle, SUB_FRAMESIZE, SEEK_CUR);
-			}
-		}
-		else {
-			s = fread(sndbuffer, 1, sizeof(sndbuffer), cddaHandle);
+			if (subChanRaw) DecodeRawSubData();
 		}
 
 		if (s == 0) {
@@ -204,10 +203,37 @@ static void *playthread(void *param)
 				}
 			}
 
-			SPU_playCDDAchannel((short *)sndbuffer, s);
+			do {
+				ret = SPU_playCDDAchannel((short *)sndbuffer, s);
+				if (ret == 0x7761)
+					usleep(6 * 1000);
+			} while (ret == 0x7761 && playing); // rearmed_wait
+		}
+
+		if (ret != 0x676f) { // !rearmed_go
+			// do approx sleep
+			long now;
+
+			// HACK: stop feeding data while emu is paused
+			extern int stop;
+			while (stop && playing)
+				usleep(10000);
+
+			now = GetTickCount();
+			osleep = t - now;
+			if (osleep <= 0) {
+				osleep = 1;
+				t = now;
+			}
+			else if (osleep > CDDA_FRAMETIME) {
+				osleep = CDDA_FRAMETIME;
+				t = now;
+			}
+
+			usleep(osleep * 1000);
+			t += CDDA_FRAMETIME;
 		}
 
-		cddaCurOffset += s;
 	}
 
 #ifdef _WIN32
@@ -230,23 +256,15 @@ static void stopCDDA() {
 #else
 	pthread_join(threadid, NULL);
 #endif
-
-	initial_offset = 0;
 }
 
 // start the CDDA playback
-static void startCDDA(unsigned int offset) {
+static void startCDDA(unsigned int sector) {
 	if (playing) {
-		if (initial_offset == offset) {
-			return;
-		}
 		stopCDDA();
 	}
 
-	initial_offset = offset;
-	cddaCurOffset = initial_offset;
-	fseek(cddaHandle, initial_offset, SEEK_SET);
-
+	cdda_cur_sector = sector;
 	playing = TRUE;
 
 #ifdef _WIN32
@@ -264,7 +282,7 @@ static int parsetoc(const char *isofile) {
 	char			linebuf[256], dummy[256], name[256];
 	char			*token;
 	char			time[20], time2[20];
-	unsigned int	t;
+	unsigned int	t, sector_offs;
 
 	numtracks = 0;
 
@@ -300,6 +318,8 @@ static int parsetoc(const char *isofile) {
 	memset(&ti, 0, sizeof(ti));
 	cddaBigEndian = TRUE; // cdrdao uses big-endian for CD Audio
 
+	sector_offs = 2 * 75;
+
 	// parse the .toc file
 	while (fgets(linebuf, sizeof(linebuf), fi) != NULL) {
 		// search for tracks
@@ -332,7 +352,8 @@ static int parsetoc(const char *isofile) {
 			if (ti[numtracks].type == CDDA) {
 				sscanf(linebuf, "DATAFILE \"%[^\"]\" #%d %8s", name, &t, time2);
 				t /= CD_FRAMESIZE_RAW + (subChanMixed ? SUB_FRAMESIZE : 0);
-				t += 2 * 75;
+				ti[numtracks].start_offset = t;
+				t += sector_offs;
 				sec2msf(t, (char *)&ti[numtracks].start);
 				tok2msf((char *)&time2, (char *)&ti[numtracks].length);
 			}
@@ -345,10 +366,20 @@ static int parsetoc(const char *isofile) {
 			sscanf(linebuf, "FILE \"%[^\"]\" #%d %8s %8s", name, &t, time, time2);
 			tok2msf((char *)&time, (char *)&ti[numtracks].start);
 			t /= CD_FRAMESIZE_RAW + (subChanMixed ? SUB_FRAMESIZE : 0);
-			t += msf2sec(ti[numtracks].start) + 2 * 75;
+			ti[numtracks].start_offset = t;
+			t += msf2sec(ti[numtracks].start) + sector_offs;
 			sec2msf(t, (char *)&ti[numtracks].start);
 			tok2msf((char *)&time2, (char *)&ti[numtracks].length);
 		}
+		else if (!strcmp(token, "ZERO")) {
+			sscanf(linebuf, "ZERO AUDIO RW_RAW %8s", time);
+			tok2msf((char *)&time, dummy);
+			sector_offs += msf2sec(dummy);
+			if (numtracks > 1) {
+				t = ti[numtracks - 1].start_offset;
+				pregapOffset = t + msf2sec(ti[numtracks - 1].length);
+			}
+		}
 	}
 
 	fclose(fi);
@@ -366,9 +397,9 @@ static int parsecue(const char *isofile) {
 	char			*token;
 	char			time[20];
 	char			*tmp;
-	char			linebuf[256], dummy[256];
+	char			linebuf[256], tmpb[256], dummy[256];
 	unsigned int	incue_max_len;
-	unsigned int	t, i, total, file_len;
+	unsigned int	t, file_len, sector_offs;
 
 	numtracks = 0;
 
@@ -413,6 +444,9 @@ static int parsecue(const char *isofile) {
 
 	memset(&ti, 0, sizeof(ti));
 
+	file_len = 0;
+	sector_offs = 2 * 75;
+
 	while (fgets(linebuf, sizeof(linebuf), fi) != NULL) {
 		strncpy(dummy, linebuf, sizeof(linebuf));
 		token = strtok(dummy, " ");
@@ -421,7 +455,7 @@ static int parsecue(const char *isofile) {
 			continue;
 		}
 
-		if (!strcmp(token, "TRACK")){
+		if (!strcmp(token, "TRACK")) {
 			numtracks++;
 
 			if (strstr(linebuf, "AUDIO") != NULL) {
@@ -432,46 +466,67 @@ static int parsecue(const char *isofile) {
 			}
 		}
 		else if (!strcmp(token, "INDEX")) {
-			tmp = strstr(linebuf, "INDEX");
-			if (tmp != NULL) {
-				tmp += strlen("INDEX") + 3; // 3 - space + numeric index
-				while (*tmp == ' ') tmp++;
-				if (*tmp != '\n') sscanf(tmp, "%8s", time);
-			}
+			sscanf(linebuf, " INDEX %02d %8s", &t, time);
+			tok2msf(time, (char *)&ti[numtracks].start);
 
-			tok2msf((char *)&time, (char *)&ti[numtracks].start);
-
-			t = msf2sec(ti[numtracks].start) + 2 * 75;
+			t = msf2sec(ti[numtracks].start);
+			ti[numtracks].start_offset = t;
+			t += sector_offs;
 			sec2msf(t, ti[numtracks].start);
+
+			// default track length to file length
+			t = file_len - ti[numtracks].start_offset;
+			sec2msf(t, ti[numtracks].length);
+
+			if (numtracks > 1 && ti[numtracks].handle == NULL) {
+				// this track uses the same file as the last,
+				// start of this track is last track's end
+				t = msf2sec(ti[numtracks].start) - msf2sec(ti[numtracks - 1].start);
+				sec2msf(t, ti[numtracks - 1].length);
+			}
+			if (numtracks > 1 && pregapOffset == -1)
+				pregapOffset = ti[numtracks].start_offset;
+		}
+		else if (!strcmp(token, "PREGAP")) {
+			if (sscanf(linebuf, " PREGAP %8s", time) == 1) {
+				tok2msf(time, dummy);
+				sector_offs += msf2sec(dummy);
+			}
+			pregapOffset = -1; // mark to fill track start_offset
 		}
 		else if (!strcmp(token, "FILE")) {
-			tmp = strstr(linebuf, "FILE");
-			if (tmp == NULL) {
-				continue;
-			}
-			tmp += 4;
-			while (*tmp == ' ')
-				tmp++;
-			if (*tmp == '"') {
-				token = tmp + 1;
-				tmp = strchr(token, '"');
+			sscanf(linebuf, " FILE \"%[^\"]\"", tmpb);
+
+			// absolute path?
+			ti[numtracks + 1].handle = fopen(tmpb, "rb");
+			if (ti[numtracks + 1].handle == NULL) {
+				// relative to .cue?
+				tmp = strrchr(tmpb, '\\');
+				if (tmp == NULL)
+					tmp = strrchr(tmpb, '/');
 				if (tmp != NULL)
-					*tmp = 0;
-			}
-			else {
-				token = tmp;
-				tmp = strchr(token, ' ');
-				if (tmp != NULL)
-					*tmp = 0;
+					tmp++;
+				else
+					tmp = tmpb;
+				strncpy(incue_fname, tmp, incue_max_len);
+				ti[numtracks + 1].handle = fopen(filepath, "rb");
 			}
 
-			strncpy(incue_fname, token, incue_max_len);
-			ti[numtracks + 1].handle = fopen(filepath, "rb");
+			// update global offset if this is not first file in this .cue
+			if (numtracks + 1 > 1)
+				sector_offs += file_len;
+
+			file_len = 0;
 			if (ti[numtracks + 1].handle == NULL) {
-				SysPrintf(_("could not open: %s\n"), filepath);
+				SysPrintf(_("\ncould not open: %s\n"), filepath);
+				continue;
 			}
-			else if (numtracks == 0 && strlen(isofile) >= 4 &&
-					strcmp(isofile + strlen(isofile) - 4, ".cue") == 0) {
+			fseek(ti[numtracks + 1].handle, 0, SEEK_END);
+			file_len = ftell(ti[numtracks + 1].handle) / 2352;
+
+			if (numtracks == 0 && strlen(isofile) >= 4 &&
+				strcmp(isofile + strlen(isofile) - 4, ".cue") == 0)
+			{
 				// user selected .cue as image file, use it's data track instead
 				fclose(cdHandle);
 				cdHandle = fopen(filepath, "rb");
@@ -481,29 +536,6 @@ static int parsecue(const char *isofile) {
 
 	fclose(fi);
 
-	// make corrections for multi-track .cue, fill track lengths
-	total = 2 * 75;
-	file_len = 0;
-	for (i = 1; i <= numtracks; i++) {
-		if (ti[i].handle != NULL) {
-			sec2msf(total, ti[i].start);
-			fseek(ti[i].handle, 0, SEEK_END);
-			file_len = ftell(ti[i].handle) / 2352;
-			sec2msf(file_len, ti[i].length);
-			total += file_len;
-		}
-		else {
-			// this track uses the same file as the last,
-			// start of this track is last track's end
-			if (i > 1) {
-				t = msf2sec(ti[i].start) - msf2sec(ti[i - 1].start);
-				sec2msf(t, ti[i - 1].length);
-			}
-			t = file_len - msf2sec(ti[i].start) + 2 * 75;
-			sec2msf(t, ti[i].length);
-		}
-	}
-
 	return 0;
 }
 
@@ -544,6 +576,7 @@ static int parseccd(const char *isofile) {
 		else if (!strncmp(linebuf, "INDEX 1=", 8)) {
 			sscanf(linebuf, "INDEX 1=%d", &t);
 			sec2msf(t + 2 * 75, ti[numtracks].start);
+			ti[numtracks].start_offset = t;
 
 			// If we've already seen another track, this is its end
 			if (numtracks > 1) {
@@ -643,16 +676,23 @@ static int parsemds(const char *isofile) {
 		ti[i].start[1] = fgetc(fi);
 		ti[i].start[2] = fgetc(fi);
 
-		if (i > 1) {
-			l = msf2sec(ti[i].start);
-			sec2msf(l - 2 * 75, ti[i].start); // ???
-		}
-
-		// get the track length
 		fread(&extra_offset, 1, sizeof(unsigned int), fi);
 		extra_offset = SWAP32(extra_offset);
 
-		fseek(fi, extra_offset + 4, SEEK_SET);
+		// get track start offset (in .mdf)
+		fseek(fi, offset + 0x28, SEEK_SET);
+		fread(&l, 1, sizeof(unsigned int), fi);
+		l = SWAP32(l);
+		ti[i].start_offset = l / CD_FRAMESIZE_RAW;
+
+		// get pregap
+		fseek(fi, extra_offset, SEEK_SET);
+		fread(&l, 1, sizeof(unsigned int), fi);
+		l = SWAP32(l);
+		if (l != 0 && i > 1)
+			pregapOffset = ti[i].start_offset;
+
+		// get the track length
 		fread(&l, 1, sizeof(unsigned int), fi);
 		l = SWAP32(l);
 		sec2msf(l, ti[i].length);
@@ -664,6 +704,133 @@ static int parsemds(const char *isofile) {
 	return 0;
 }
 
+static int parsepbp(const char *isofile) {
+	struct {
+		unsigned int sig;
+		unsigned int dontcare[8];
+		unsigned int psar_offs;
+	} pbp_hdr;
+	struct {
+		unsigned char type;
+		unsigned char pad0;
+		unsigned char track;
+		char index0[3];
+		char pad1;
+		char index1[3];
+	} toc_entry;
+	struct {
+		unsigned int offset;
+		unsigned int size;
+		unsigned int dontcare[6];
+	} index_entry;
+	char psar_sig[9];
+	unsigned int t, cd_length, cdimg_base;
+	const char *ext = NULL;
+	int i, ret;
+
+	if (strlen(isofile) >= 4)
+		ext = isofile + strlen(isofile) - 4;
+	if (ext == NULL || (strcmp(ext, ".pbp") != 0 && strcmp(ext, ".PBP") != 0))
+		return -1;
+
+	numtracks = 0;
+
+	ret = fread(&pbp_hdr, 1, sizeof(pbp_hdr), cdHandle);
+	if (ret != sizeof(pbp_hdr)) {
+		fprintf(stderr, "failed to read pbp\n");
+		goto fail_io;
+	}
+
+	ret = fseek(cdHandle, pbp_hdr.psar_offs, SEEK_SET);
+	if (ret != 0) {
+		fprintf(stderr, "failed to seek to %x\n", pbp_hdr.psar_offs);
+		goto fail_io;
+	}
+
+	fread(psar_sig, 1, sizeof(psar_sig), cdHandle);
+	psar_sig[8] = 0;
+	if (strcmp(psar_sig, "PSISOIMG") != 0) {
+		fprintf(stderr, "bad psar_sig: %s\n", psar_sig);
+		goto fail_io;
+	}
+
+	// seek to TOC
+	ret = fseek(cdHandle, pbp_hdr.psar_offs + 0x800, SEEK_SET);
+	if (ret != 0) {
+		fprintf(stderr, "failed to seek to %x\n", pbp_hdr.psar_offs);
+		goto fail_io;
+	}
+
+	// first 3 entries are special
+	fseek(cdHandle, sizeof(toc_entry), SEEK_CUR);
+	fread(&toc_entry, 1, sizeof(toc_entry), cdHandle);
+	numtracks = btoi(toc_entry.index1[0]);
+
+	fread(&toc_entry, 1, sizeof(toc_entry), cdHandle);
+	cd_length = btoi(toc_entry.index1[0]) * 60 * 75 +
+		btoi(toc_entry.index1[1]) * 75 + btoi(toc_entry.index1[2]);
+
+	for (i = 1; i <= numtracks; i++) {
+		fread(&toc_entry, 1, sizeof(toc_entry), cdHandle);
+
+		ti[i].type = (toc_entry.type == 1) ? CDDA : DATA;
+
+		ti[i].start_offset = btoi(toc_entry.index0[0]) * 60 * 75 +
+			btoi(toc_entry.index0[1]) * 75 + btoi(toc_entry.index0[2]);
+		ti[i].start[0] = btoi(toc_entry.index1[0]);
+		ti[i].start[1] = btoi(toc_entry.index1[1]);
+		ti[i].start[2] = btoi(toc_entry.index1[2]);
+
+		if (i > 1) {
+			t = msf2sec(ti[i].start) - msf2sec(ti[i - 1].start);
+			sec2msf(t, ti[i - 1].length);
+		}
+	}
+	t = cd_length - ti[numtracks].start_offset;
+	sec2msf(t, ti[numtracks].length);
+
+	// seek to ISO index
+	ret = fseek(cdHandle, pbp_hdr.psar_offs + 0x4000, SEEK_SET);
+	if (ret != 0) {
+		fprintf(stderr, "failed to seek to ISO index\n");
+		goto fail_io;
+	}
+
+	compr_img = calloc(1, sizeof(*compr_img));
+	if (compr_img == NULL)
+		goto fail_io;
+
+	compr_img->current_block = (unsigned int)-1;
+
+	compr_img->index_len = (0x100000 - 0x4000) / sizeof(index_entry);
+	compr_img->index_table = malloc((compr_img->index_len + 1) * sizeof(compr_img->index_table[0]));
+	if (compr_img->index_table == NULL)
+		goto fail_io;
+
+	cdimg_base = pbp_hdr.psar_offs + 0x100000;
+	for (i = 0; i < compr_img->index_len; i++) {
+		ret = fread(&index_entry, 1, sizeof(index_entry), cdHandle);
+		if (ret != sizeof(index_entry)) {
+			fprintf(stderr, "failed to read index_entry #%d\n", i);
+			goto fail_index;
+		}
+
+		if (index_entry.size == 0)
+			break;
+
+		compr_img->index_table[i] = cdimg_base + index_entry.offset;
+	}
+	compr_img->index_table[i] = cdimg_base + index_entry.offset + index_entry.size;
+
+	return 0;
+
+fail_index:
+	free(compr_img->index_table);
+	compr_img->index_table = NULL;
+fail_io:
+	return -1;
+}
+
 // this function tries to get the .sub file of the given .img
 static int opensubfile(const char *isoname) {
 	char		subname[MAXPATHLEN];
@@ -705,6 +872,139 @@ static int opensbifile(const char *isoname) {
 	return LoadSBI(sbiname, s);
 }
 
+static int cdread_normal(FILE *f, void *dest, int sector, int offset)
+{
+	fseek(f, sector * CD_FRAMESIZE_RAW + offset, SEEK_SET);
+	return fread(dest, 1, CD_FRAMESIZE_RAW - offset, f);
+}
+
+static int cdread_sub_mixed(FILE *f, void *dest, int sector, int offset)
+{
+	int ret;
+
+	fseek(f, sector * (CD_FRAMESIZE_RAW + SUB_FRAMESIZE) + offset, SEEK_SET);
+	ret = fread(dest, 1, CD_FRAMESIZE_RAW - offset, f);
+	fread(subbuffer, 1, SUB_FRAMESIZE, f);
+
+	if (subChanRaw) DecodeRawSubData();
+
+	return ret;
+}
+
+static int uncompress2(void *out, unsigned long *out_size, void *in, unsigned long in_size)
+{
+	static z_stream z;
+	int ret = 0;
+
+	if (z.zalloc == NULL) {
+		// XXX: one-time leak here..
+		z.next_in = Z_NULL;
+		z.avail_in = 0;
+		z.zalloc = Z_NULL;
+		z.zfree = Z_NULL;
+		z.opaque = Z_NULL;
+		ret = inflateInit2(&z, -15);
+	}
+	else
+		ret = inflateReset(&z);
+	if (ret != Z_OK)
+		return ret;
+
+	z.next_in = in;
+	z.avail_in = in_size;
+	z.next_out = out;
+	z.avail_out = *out_size;
+
+	ret = inflate(&z, Z_NO_FLUSH);
+	//inflateEnd(&z);
+
+	*out_size -= z.avail_out;
+	return ret == 1 ? 0 : ret;
+}
+
+static int cdread_compressed(FILE *f, void *dest, int sector, int offset)
+{
+	unsigned int start_byte, size;
+	unsigned long cdbuffer_size;
+	int ret, block;
+
+	block = sector >> 4;
+	compr_img->sector_in_blk = sector & 15;
+
+	if (block == compr_img->current_block) {
+		//printf("hit sect %d\n", sector);
+		goto finish;
+	}
+
+	if (sector >= compr_img->index_len * 16) {
+		fprintf(stderr, "sector %d is past img end\n", sector);
+		return -1;
+	}
+
+	start_byte = compr_img->index_table[block];
+	if (fseek(cdHandle, start_byte, SEEK_SET) != 0) {
+		fprintf(stderr, "seek error for block %d at %x: ",
+			block, start_byte);
+		perror(NULL);
+		return -1;
+	}
+
+	size = compr_img->index_table[block + 1] - start_byte;
+	if (size > sizeof(compr_img->buff_compressed)) {
+		fprintf(stderr, "block %d is too large: %u\n", block, size);
+		return -1;
+	}
+
+	if (fread(compr_img->buff_compressed, 1, size, cdHandle) != size) {
+		fprintf(stderr, "read error for block %d at %x: ", block, start_byte);
+		perror(NULL);
+		return -1;
+	}
+
+	cdbuffer_size = sizeof(compr_img->buff_raw[0]) * 16;
+	ret = uncompress2(compr_img->buff_raw[0], &cdbuffer_size, compr_img->buff_compressed, size);
+	if (ret != 0) {
+		fprintf(stderr, "uncompress failed with %d for block %d, sector %d\n",
+			ret, block, sector);
+		return -1;
+	}
+	if (cdbuffer_size != sizeof(compr_img->buff_raw[0]) * 16)
+		fprintf(stderr, "cdbuffer_size: %lu != %d, sector %d\n", cdbuffer_size,
+			sizeof(compr_img->buff_raw[0]) * 16, sector);
+
+	// done at last!
+	compr_img->current_block = block;
+
+finish:
+	if (dest != cdbuffer) // copy avoid HACK
+		memcpy(dest, compr_img->buff_raw[compr_img->sector_in_blk] + offset,
+			CD_FRAMESIZE_RAW - offset);
+	return CD_FRAMESIZE_RAW - offset;
+}
+
+static int cdread_2048(FILE *f, void *dest, int sector, int offset)
+{
+	int ret;
+
+	fseek(f, sector * 2048, SEEK_SET);
+	ret = fread((char *)dest + 12, 1, 2048, f);
+
+	// not really necessary, fake mode 2 header
+	memset(cdbuffer, 0, 12);
+	sec2msf(sector + 2 * 75, (char *)cdbuffer);
+	cdbuffer[3] = 1;
+
+	return ret;
+}
+
+static unsigned char * CALLBACK ISOgetBuffer_compr(void) {
+	return compr_img->buff_raw[compr_img->sector_in_blk] + 12;
+}
+
+static unsigned char * CALLBACK ISOgetBuffer(void) {
+	return cdbuffer;
+}
+
 static void PrintTracks(void) {
 	int i;
 
@@ -719,6 +1019,8 @@ static void PrintTracks(void) {
 // This function is invoked by the front-end when opening an ISO
 // file for playback
 static long CALLBACK ISOopen(void) {
+	boolean isMode1ISO = FALSE;
+
 	if (cdHandle != NULL) {
 		return 0; // it's already open
 	}
@@ -733,6 +1035,10 @@ static long CALLBACK ISOopen(void) {
 	cddaBigEndian = FALSE;
 	subChanMixed = FALSE;
 	subChanRaw = FALSE;
+	pregapOffset = 0;
+
+	CDR_getBuffer = ISOgetBuffer;
+	cdimg_read_func = cdread_normal;
 
 	if (parsecue(GetIsoFile()) == 0) {
 		SysPrintf("[+cue]");
@@ -746,6 +1052,11 @@ static long CALLBACK ISOopen(void) {
 	else if (parsemds(GetIsoFile()) == 0) {
 		SysPrintf("[+mds]");
 	}
+	else if (parsepbp(GetIsoFile()) == 0) {
+		SysPrintf("[pbp]");
+		CDR_getBuffer = ISOgetBuffer_compr;
+		cdimg_read_func = cdread_compressed;
+	}
 
 	if (!subChanMixed && opensubfile(GetIsoFile()) == 0) {
 		SysPrintf("[+sub]");
@@ -754,15 +1065,33 @@ static long CALLBACK ISOopen(void) {
 		SysPrintf("[+sbi]");
 	}
 
+	// guess whether it is mode1/2048
+	fseek(cdHandle, 0, SEEK_END);
+	if (ftell(cdHandle) % 2048 == 0) {
+		unsigned int modeTest = 0;
+		fseek(cdHandle, 0, SEEK_SET);
+		fread(&modeTest, 4, 1, cdHandle);
+		if (SWAP32(modeTest) != 0xffffff00) {
+			SysPrintf("[2048]");
+			isMode1ISO = TRUE;
+		}
+	}
+	fseek(cdHandle, 0, SEEK_SET);
+
 	SysPrintf(".\n");
 
 	PrintTracks();
 
+	if (subChanMixed)
+		cdimg_read_func = cdread_sub_mixed;
+	else if (isMode1ISO)
+		cdimg_read_func = cdread_2048;
+
 	// make sure we have another handle open for cdda
 	if (numtracks > 1 && ti[1].handle == NULL) {
 		ti[1].handle = fopen(GetIsoFile(), "rb");
 	}
-	cddaCurOffset = cddaStartOffset = 0;
+	cdda_cur_sector = cdda_start_sector = 0;
 
 	return 0;
 }
@@ -781,6 +1110,12 @@ static long CALLBACK ISOclose(void) {
 	stopCDDA();
 	cddaHandle = NULL;
 
+	if (compr_img != NULL) {
+		free(compr_img->index_table);
+		free(compr_img);
+		compr_img = NULL;
+	}
+
 	for (i = 1; i <= numtracks; i++) {
 		if (ti[i].handle != NULL) {
 			fclose(ti[i].handle);
@@ -833,7 +1168,7 @@ static long CALLBACK ISOgetTD(unsigned char track, unsigned char *buffer) {
 		unsigned int sect;
 		unsigned char time[3];
 		sect = msf2sec(ti[numtracks].start) + msf2sec(ti[numtracks].length);
-		sec2msf(sect, time);
+		sec2msf(sect, (char *)time);
 		buffer[2] = time[0];
 		buffer[1] = time[1];
 		buffer[0] = time[2];
@@ -872,68 +1207,68 @@ static void DecodeRawSubData(void) {
 // time: byte 0 - minute; byte 1 - second; byte 2 - frame
 // uses bcd format
 static long CALLBACK ISOreadTrack(unsigned char *time) {
+	int sector = MSF2SECT(btoi(time[0]), btoi(time[1]), btoi(time[2]));
+
 	if (cdHandle == NULL) {
 		return -1;
 	}
 
-	if (subChanMixed) {
-		fseek(cdHandle, MSF2SECT(btoi(time[0]), btoi(time[1]), btoi(time[2])) * (CD_FRAMESIZE_RAW + SUB_FRAMESIZE) + 12, SEEK_SET);
-		fread(cdbuffer, 1, DATA_SIZE, cdHandle);
-		fread(subbuffer, 1, SUB_FRAMESIZE, cdHandle);
-
-		if (subChanRaw) DecodeRawSubData();
-	}
-	else {
-		fseek(cdHandle, MSF2SECT(btoi(time[0]), btoi(time[1]), btoi(time[2])) * CD_FRAMESIZE_RAW + 12, SEEK_SET);
-		fread(cdbuffer, 1, DATA_SIZE, cdHandle);
-
-		if (subHandle != NULL) {
-			fseek(subHandle, MSF2SECT(btoi(time[0]), btoi(time[1]), btoi(time[2])) * SUB_FRAMESIZE, SEEK_SET);
-			fread(subbuffer, 1, SUB_FRAMESIZE, subHandle);
-
-			if (subChanRaw) DecodeRawSubData();
+	if (pregapOffset) {
+		subChanMissing = FALSE;
+		if (sector >= pregapOffset) {
+			sector -= 2 * 75;
+			if (sector < pregapOffset)
+				subChanMissing = TRUE;
 		}
 	}
 
-	return 0;
-}
+	cdimg_read_func(cdHandle, cdbuffer, sector, 12);
 
-// return readed track
-static unsigned char * CALLBACK ISOgetBuffer(void) {
-	return cdbuffer;
+	if (subHandle != NULL) {
+		fseek(subHandle, sector * SUB_FRAMESIZE, SEEK_SET);
+		fread(subbuffer, 1, SUB_FRAMESIZE, subHandle);
+
+		if (subChanRaw) DecodeRawSubData();
+	}
+
+	return 0;
 }
 
 // plays cdda audio
 // sector: byte 0 - minute; byte 1 - second; byte 2 - frame
 // does NOT uses bcd format
 static long CALLBACK ISOplay(unsigned char *time) {
-	unsigned int i, sect;
+	unsigned int i, abs_sect;
+	int file_sect;
 
 	if (numtracks <= 1)
 		return 0;
 
 	// find the track
-	sect = msf2sec(time);
+	abs_sect = msf2sec((char *)time);
 	for (i = numtracks; i > 1; i--)
-		if (msf2sec(ti[i].start) <= sect + 2 * 75)
+		if (msf2sec(ti[i].start) <= abs_sect + 2 * 75)
 			break;
 
+	file_sect = ti[i].start_offset + (abs_sect - msf2sec(ti[i].start));
+	if (file_sect < 0)
+		file_sect = 0;
+
 	// find the file that contains this track
 	for (; i > 1; i--)
 		if (ti[i].handle != NULL)
 			break;
 
-	cddaStartOffset = msf2sec(ti[i].start);
-	sect -= cddaStartOffset - 2 * 75;
+	cdda_start_sector = abs_sect - file_sect;
 	cddaHandle = ti[i].handle;
 
+	if (pregapOffset && (unsigned int)(pregapOffset - file_sect) < 2 * 75) {
+		// get out of the missing pregap to avoid noise
+		file_sect = pregapOffset;
+	}
+
 	if (SPU_playCDDAchannel != NULL) {
-		if (subChanMixed) {
-			startCDDA(sect * (CD_FRAMESIZE_RAW + SUB_FRAMESIZE));
-		}
-		else {
-			startCDDA(sect * CD_FRAMESIZE_RAW);
-		}
+		startCDDA(file_sect);
 	}
 	return 0;
 }
@@ -946,7 +1281,7 @@ static long CALLBACK ISOstop(void) {
 
 // gets subchannel data
 static unsigned char* CALLBACK ISOgetBufferSub(void) {
-	if (subHandle != NULL || subChanMixed) {
+	if ((subHandle != NULL || subChanMixed) && !subChanMissing) {
 		return subbuffer;
 	}
 
@@ -966,7 +1301,7 @@ static long CALLBACK ISOgetStatus(struct CdrStat *stat) {
 		stat->Type = 0x01;
 	}
 
-	sec = (cddaStartOffset + cddaCurOffset) / CD_FRAMESIZE_RAW;
+	sec = cdda_start_sector + cdda_cur_sector;
 	sec2msf(sec, (char *)stat->Time);
 
 	return 0;
