@@ -709,11 +709,12 @@ void cdrInterrupt() {
 			Wild 9: skip PREGAP + starting accurate SubQ
 			- plays tracks without retry play
 			*/
+			/* unneeded with correct cdriso?
 			Set_Track();
+			*/
 			Find_CurTrack();
 			ReadTrack( cdr.SetSectorPlay );
 
-
 			// GameShark CD Player: Calls 2x + Play 2x
 			if( cdr.FastBackward || cdr.FastForward ) {
 				if( cdr.FastForward ) cdr.FastForward--;
@@ -1216,7 +1217,8 @@ void cdrInterrupt() {
 			// - fixes cutscene speech
 			{
 				u8 *buf = CDR_getBuffer();
-				memcpy(cdr.Transfer, buf, 8);
+				if (buf != NULL)
+					memcpy(cdr.Transfer, buf, 8);
 			}
 			
 			
@@ -1327,6 +1329,23 @@ void cdrReadInterrupt() {
 			int ret = xa_decode_sector(&cdr.Xa, cdr.Transfer+4, cdr.FirstSector);
 
 			if (!ret) {
+				// only handle attenuator basic channel switch for now
+				if (cdr.Xa.stereo) {
+					int i;
+					if ((cdr.AttenuatorLeft[0] | cdr.AttenuatorLeft[1])
+					    && !(cdr.AttenuatorRight[0] | cdr.AttenuatorRight[1]))
+					{
+						for (i = 0; i < cdr.Xa.nsamples; i++)
+							cdr.Xa.pcm[i*2 + 1] = cdr.Xa.pcm[i*2];
+					}
+					else if (!(cdr.AttenuatorLeft[0] | cdr.AttenuatorLeft[1])
+					    && (cdr.AttenuatorRight[0] | cdr.AttenuatorRight[1]))
+					{
+						for (i = 0; i < cdr.Xa.nsamples; i++)
+							cdr.Xa.pcm[i*2] = cdr.Xa.pcm[i*2 + 1];
+					}
+				}
+
 				SPU_playADPCMchannel(&cdr.Xa);
 				cdr.FirstSector = 0;
 
@@ -1463,7 +1482,7 @@ void cdrWrite1(unsigned char rt) {
 
 	// Tekken: CDXA fade-out
 	if( (cdr.Ctrl & 3) == 3 ) {
-		//cdr.AttenuatorRight[0] = rt;
+		cdr.AttenuatorRight[0] = rt;
 	}
 
 
@@ -1815,10 +1834,10 @@ void cdrWrite2(unsigned char rt) {
 
 	// Tekken: CDXA fade-out
 	if( (cdr.Ctrl & 3) == 2 ) {
-		//cdr.AttenuatorLeft[0] = rt;
+		cdr.AttenuatorLeft[0] = rt;
 	}
 	else if( (cdr.Ctrl & 3) == 3 ) {
-		//cdr.AttenuatorRight[1] = rt;
+		cdr.AttenuatorRight[1] = rt;
 	}
 
 
@@ -1860,7 +1879,7 @@ void cdrWrite3(unsigned char rt) {
 #ifdef CDR_LOG
 	CDR_LOG("cdrWrite3() Log: CD3 write: %x\n", rt);
 #endif
-/*
+
 	// Tekken: CDXA fade-out
 	if( (cdr.Ctrl & 3) == 2 ) {
 		cdr.AttenuatorLeft[1] = rt;
@@ -1872,7 +1891,7 @@ void cdrWrite3(unsigned char rt) {
 			cdr.AttenuatorRight[0], cdr.AttenuatorRight[1] );
 #endif
 	}
-*/
+
 
 	// GameShark CDX CD Player: Irq timing mania
 	if( rt == 0 &&
@@ -1900,9 +1919,14 @@ void cdrWrite3(unsigned char rt) {
 		// - Final Fantasy Tactics
 		// - various other games
 
-		if (cdr.Irq) // rearmed guesswork hack
 		if (cdr.Reading && !cdr.ResultReady) {
-			CDREAD_INT((cdr.Mode & MODE_SPEED) ? (cdReadTime / 2) : cdReadTime);
+			int left = psxRegs.intCycle[PSXINT_CDREAD].sCycle + psxRegs.intCycle[PSXINT_CDREAD].cycle - psxRegs.cycle;
+			int time = (cdr.Mode & MODE_SPEED) ? (cdReadTime / 2) : cdReadTime;
+			if (Config.CdrReschedule != 2)
+			if (left < time / 2 || Config.CdrReschedule) { // rearmed guesswork hack
+				//printf("-- resched %d -> %d\n", left, time);
+				CDREAD_INT(time);
+			}
 		}
 
 		return;
@@ -1930,6 +1954,7 @@ void cdrWrite3(unsigned char rt) {
 
 void psxDma3(u32 madr, u32 bcr, u32 chcr) {
 	u32 cdsize;
+	int size;
 	u8 *ptr;
 
 #ifdef CDR_LOG
@@ -1974,15 +1999,12 @@ void psxDma3(u32 madr, u32 bcr, u32 chcr) {
 			- CdlPlay
 			- Spams DMA3 and gets buffer overrun
 			*/
-
-			if( (cdr.pTransfer-cdr.Transfer) + cdsize > 2352 )
+			size = CD_FRAMESIZE_RAW - (cdr.pTransfer - cdr.Transfer);
+			if (size > cdsize)
+				size = cdsize;
+			if (size > 0)
 			{
-				// avoid crash - probably should wrap here
-				//memcpy(ptr, cdr.pTransfer, cdsize);
-			}
-			else
-			{
-				memcpy(ptr, cdr.pTransfer, cdsize);
+				memcpy(ptr, cdr.pTransfer, size);
 			}
 
 			psxCpu->Clear(madr, cdsize / 4);
@@ -2011,8 +2033,11 @@ void psxDma3(u32 madr, u32 bcr, u32 chcr) {
 
 void cdrDmaInterrupt()
 {
-	HW_DMA3_CHCR &= SWAP32(~0x01000000);
-	DMA_INTERRUPT(3);
+	if (HW_DMA3_CHCR & SWAP32(0x01000000))
+	{
+		HW_DMA3_CHCR &= SWAP32(~0x01000000);
+		DMA_INTERRUPT(3);
+	}
 }
 
 void cdrReset() {
@@ -2020,6 +2045,12 @@ void cdrReset() {
 	cdr.CurTrack = 1;
 	cdr.File = 1;
 	cdr.Channel = 1;
+
+	// BIOS player - default values
+	cdr.AttenuatorLeft[0] = 0x80;
+	cdr.AttenuatorLeft[1] = 0x00;
+	cdr.AttenuatorRight[0] = 0x80;
+	cdr.AttenuatorRight[1] = 0x00;
 }
 
 int cdrFreeze(gzFile f, int Mode) {
