@@ -20,19 +20,19 @@
 #define D_WIDTH			640
 #define D_HEIGHT		480
 
+int g_layer_x = (X_RES - D_WIDTH) / 2;
+int g_layer_y = (Y_RES - D_HEIGHT) / 2;
+int g_layer_w = D_WIDTH, g_layer_h = D_HEIGHT;
+
 static GdkImage *image;
 static HildonAnimationActor *actor;
 static GtkWidget *window, *drawing;
 
-void *pl_fbdev_buf;
 int keystate;
-int g_opts=0;
-int sens,y_def;
-extern int gles;
 int pl_buf_w=0,pl_buf_h=0;
+extern int g_opts;
+int sens,y_def;
 extern char * file_name;
-
-int in_type = PSE_PAD_TYPE_STANDARD;
 int in_keystate, in_a1[2], in_a2[2];
 static int keymap[65536];
 
@@ -60,9 +60,6 @@ void hildon_quit()
 	exit(0);
 }
 
-
-#include "xkb.c"
-
 static void
 window_key_proxy(GtkWidget *widget,
 		     GdkEventKey *event,
@@ -98,17 +95,17 @@ window_key_proxy(GtkWidget *widget,
 			break;
 		case 19:
 			if (event->type == GDK_KEY_PRESS)
-	{
+		{
 				emu_save_state(state_slot);
-				
 				char buf[MAXPATHLEN];
-				sprintf (buf,"/opt/maemo/usr/games/screenshots/%s.%3.3d",file_name,state_slot);
+				sprintf (buf,"/opt/maemo/usr/games/screenshots%s.%3.3d",file_name,state_slot);
 				writepng(buf, image->mem, pl_buf_w,pl_buf_h);
 
 			return;
 	}
 		case 20:
 			if (event->type == GDK_KEY_PRESS)
+
 				emu_load_state(state_slot);
 			return;
 		case 21:
@@ -151,7 +148,7 @@ void maemo_init(int *argc, char ***argv)
 	for (i=0;i<65536;i++)
 		keymap[i]=164;
 	if (NULL != pFile) {
-		for(i=0;i<23;i++){
+		for(i=0;i<21;i++){
 			fscanf(pFile, "%i",&ch);
 			keymap[ch]=i;
 		}
@@ -167,8 +164,6 @@ void maemo_init(int *argc, char ***argv)
 	y_def=500; //near 45 degrees =)
 	}
 	
-	if (gles!=0){
-	return;}
 	gtk_init (argc, argv);
 
 	window = hildon_stackable_window_new ();
@@ -202,21 +197,20 @@ void menu_loop(void)
 {
 }
 
-void *pl_fbdev_set_mode(int w, int h, int bpp)
+void *hildon_set_mode(int w, int h)
 {
-	if (gles!=0)return pl_fbdev_buf;
 	if (w <= 0 || h <= 0)
-		return pl_fbdev_buf;
+		return pl_vout_buf;
 
 	if (image) gdk_image_destroy(image);
 	image = gdk_image_new( GDK_IMAGE_FASTEST, gdk_visual_get_system(), w, h );
 
-	pl_fbdev_buf = (void *) image->mem;
+	pl_vout_buf = (void *) image->mem;
 
 	gtk_image_set_from_image (GTK_IMAGE(drawing), image, NULL);
 
 	gtk_window_resize (GTK_WINDOW (actor), w, h);
-	if (g_opts & 2)
+if (g_opts & 2)
 	hildon_animation_actor_set_scale (actor,
 				(gdouble)800 / (gdouble)w,
 				(gdouble)480 / (gdouble)h
@@ -226,26 +220,16 @@ else
 				(gdouble)D_WIDTH / (gdouble)w,
 				(gdouble)D_HEIGHT / (gdouble)h
 	);
- pl_buf_w=w;pl_buf_h=h;
-return pl_fbdev_buf;
+	pl_buf_w=w;pl_buf_h=h;
+	return pl_vout_buf;
 }
-void *pl_fbdev_flip(void)
-{
-	if (gles!=0)return pl_fbdev_buf;
-	gtk_widget_queue_draw (drawing);
-	return pl_fbdev_buf;
-}
-void pl_frame_limit(void)
-{
-	
-	//printf("gp2x_joystick_read\n");
-	/* process GTK+ events */
-		
-if (gles==1)read_x11_keys();else
-	while (gtk_events_pending())
-		gtk_main_iteration();
 
-if (g_opts&4){
+void *hildon_flip(void)
+{
+	gtk_widget_queue_draw (drawing);
+	
+// process accelometer
+	if (g_opts&4){
 	int x, y, z;
 	FILE* f = fopen( "/sys/class/i2c-adapter/i2c-3/3-001d/coord", "r" );
 if( !f ) {printf ("err in accel");return;}
@@ -262,45 +246,19 @@ if( !f ) {printf ("err in accel");return;}
 		else {keystate &= ~(1 << DKEY_DOWN);keystate &= ~(1 << DKEY_UP);}
 
 }
+	/* process GTK+ events */
+	while (gtk_events_pending())
+		gtk_main_iteration();
 
-
+	return pl_vout_buf;
 }
 
-
-void pl_fbdev_close(void)
-{
-}
-
-int pl_fbdev_open(void)
+int omap_enable_layer(int enabled)
 {
 	return 0;
 }
 
-static void pl_get_layer_pos(int *x, int *y, int *w, int *h)
+void menu_notify_mode_change(int w, int h, int bpp)
 {
-	*x = 0;
-	*y = 0;
-	*w = 800;
-	*h = 640;
-}
-
-extern int UseFrameSkip; // hmh
-
-const struct rearmed_cbs pl_rearmed_cbs = {
-	pl_get_layer_pos,
-	pl_fbdev_open,
-	pl_fbdev_set_mode,
-	pl_fbdev_flip,
-	pl_fbdev_close,
-	&UseFrameSkip,
-};
-
-void *pl_prepare_screenshot(int *w, int *h, int *bpp)
-{
-    *w = 800;
-	*h = 640;
-	*bpp=16;
-	
-	return pl_fbdev_buf;
 }
 
