@@ -7,17 +7,15 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <stdarg.h>
-#include <dlfcn.h>
-#include <sys/stat.h>
-#include <sys/types.h>
+#include <stdint.h>
 #include <unistd.h>
-#include <signal.h>
 
 #include "main.h"
+#include "menu.h"
 #include "plugin.h"
+#include "plugin_lib.h"
 #include "../libpcsxcore/misc.h"
-#include "../plugins/cdrcimg/cdrcimg.h"
+#include "../libpcsxcore/new_dynarec/new_dynarec.h"
 
 // from softgpu plugin
 extern int iUseDither;
@@ -33,15 +31,19 @@ extern int iUseInterpolation;
 extern int iXAPitch;
 extern int iSPUIRQWait;
 extern int iUseTimer;
-extern int g_opts;
-int gles=0;
-char * file_name;
-  
+
+int g_opts = OPT_SHOWFPS;
+
+enum sched_action emu_action;
+void do_emu_action(void);
+char* file_name;
+const char *cdfile = NULL;
+
 static void ChangeWorkingDirectory(char *exe)
 {
-	s8 exepath[1024];
-	s8* s;
-	sprintf(exepath, "%s", exe);
+	char exepath[1024];
+	char *s;
+	snprintf(exepath, sizeof(exepath), "%s", exe);
 	s = strrchr(exepath, '/');
 	if (s != NULL) {
 		*s = '\0';
@@ -54,7 +56,7 @@ int maemo_main(int argc, char **argv)
 	ChangeWorkingDirectory("c");
 	char file[MAXPATHLEN] = "";
 	char path[MAXPATHLEN];
-	const char *cdfile = NULL;
+
 	int loadst = 0;
 	void *tmp;
 	int i;
@@ -63,14 +65,13 @@ int maemo_main(int argc, char **argv)
 	strcpy(Config.PluginsDir, "/opt/maemo/usr/games/plugins");
 	snprintf(Config.PatchesDir, sizeof(Config.PatchesDir), "/opt/maemo/usr/games" PATCHES_DIR);
 	Config.PsxAuto = 1;
-	char* screen;
+	
 	// read command line options
 	for (i = 1; i < argc; i++) {
 		     if (!strcmp(argv[i], "-psxout")) Config.PsxOut = 1;
 		else if (!strcmp(argv[i], "-load")) loadst = atol(argv[++i]);
 		else if (!strcmp(argv[i], "-cdfile")) {
 			char isofilename[MAXPATHLEN];
-
 			if (i+1 >= argc) break;
 			strncpy(isofilename, argv[++i], MAXPATHLEN);
 			if (isofilename[0] != '/') {
@@ -82,82 +83,58 @@ int maemo_main(int argc, char **argv)
 				} else
 					isofilename[0] = 0;
 			}
-
 			cdfile = isofilename;
 		}
-		else if (!strcmp(argv[i],"-frameskip")){
-		
-		int tv_reg=atol(argv[++i]);
-		if (tv_reg>0){
-		UseFrameSkip=1;
-		fFrameRateHz = (tv_reg==1)?50.0f:  59.94f;
-		dwFrameRateTicks = (100000*100 / (unsigned long)(fFrameRateHz*100));
-		}
+		else if (!strcmp(argv[i],"-frameskip")) {
+			int tv_reg = atol(argv[++i]);
+			if (tv_reg > 0)
+				pl_rearmed_cbs.frameskip = -1;
 		}
 		else if (!strcmp(argv[i],"-fullscreen"))		g_opts |= 2;
 		else if (!strcmp(argv[i],"-accel"))				g_opts |= 4;
 		else if (!strcmp(argv[i],"-sputhreaded"))		iUseTimer=1;
-		else if (!strcmp(argv[i],"-gles"))	{strcpy(Config.Gpu, "gpuGLES.so");gles=1;}
-		else if (!strcmp(argv[i],"-pcsx4all"))	{strcpy(Config.Gpu, "gpuPCSX4ALL.so");}
-		else if (!strcmp(argv[i],"-pcsxgpu"))	{strcpy(Config.Gpu, "gpuPCSX4ALL.so");}
-		else if(!strcmp(argv[i], "-bdir"))		sprintf(Config.BiosDir, "%s", argv[++i]);
-		else if(!strcmp(argv[i], "-bios"))		sprintf(Config.Bios, "%s", argv[++i]);
-		else if (!strcmp(argv[i],"-nosound"))	strcpy(Config.Spu, "spunull.so");
+		else if (!strcmp(argv[i],"-nosound"))		strcpy(Config.Spu, "spunull.so");
+		/* unworking with r10
+		else if(!strcmp(argv[i], "-bdir"))			sprintf(Config.BiosDir, "%s", argv[++i]);
+		else if(!strcmp(argv[i], "-bios"))			sprintf(Config.Bios, "%s", argv[++i]);
+		else if (!strcmp(argv[i],"-gles"))			strcpy(Config.Gpu, "gpuGLES.so");
+		*/
 		else if (!strcmp(argv[i], "-cdda"))		Config.Cdda = 1;
 		else if (!strcmp(argv[i], "-xa"))		Config.Xa = 1;
 		else if (!strcmp(argv[i], "-rcnt"))		Config.RCntFix = 1 ;
 		else if (!strcmp(argv[i], "-sio"))		Config.Sio = 1;
 		else if (!strcmp(argv[i], "-spuirq"))	Config.SpuIrq = 1;
 		else if (!strcmp(argv[i], "-vsync"))	Config.VSyncWA = 1;
-		else if (!strcmp(argv[i], "-h") ||
-			 !strcmp(argv[i], "-help") ||
-			 !strcmp(argv[i], "--help")) {
-			 printf(PACKAGE_NAME " " PACKAGE_VERSION "\n");
-			 printf("%s\n", _(
-							" pcsx [options] [file]\n"
-							"\toptions:\n"
-							"\t-cdfile FILE\tRuns a CD image file\n"
-							"\t-psxout\t\tEnable PSX output\n"
-							"\t-nosound\t\tDisable sound using spunull plugin\n"
-							"\t-sputhreaded\t\tMove sound to separate thread\n"
-							"\t-frameskip\t\tEnable frameskip\n"
-							"\t-load STATENUM\tLoads savestate STATENUM (1-5)\n"
-							"\t-h -help\tDisplay this message\n"
-							"\tfile\t\tLoads file\n"));
-			 return 0;
-		} else {
-			strncpy(file, argv[i], MAXPATHLEN);
-			if (file[0] != '/') {
-				getcwd(path, MAXPATHLEN);
-				if (strlen(path) + strlen(file) + 1 < MAXPATHLEN) {
-					strcat(path, "/");
-					strcat(path, file);
-					strcpy(file, path);
-				} else
-					file[0] = 0;
-			}
-		}
 	}
-	
+
+	pl_rearmed_cbs.gpu_peops.dwActFixes = 1<<7;
+	iUseReverb = 2;
+	iUseInterpolation = 1;
+	iSPUIRQWait = 1;
+	iUseTimer = 2;
+
+	in_type1 = PSE_PAD_TYPE_STANDARD;
+	in_type2 = PSE_PAD_TYPE_STANDARD;
 
 	hildon_init(&argc, &argv);
 	
-file_name=strrchr(cdfile,'/');
-file_name++;
-
-	
+			char f_name[MAXPATHLEN];
+			strcpy(f_name, strrchr(cdfile,'/'));
+			file_name=f_name;
+			printf (f_name);
 	if (cdfile)
 		set_cd_image(cdfile);
 
 	if (SysInit() == -1)
 		return 1;
 
+	pl_init();
+
 	if (LoadPlugins() == -1) {
 		SysMessage("Failed loading plugins!");
 		return 1;
 	}
 
-	pcnt_hook_plugins();
 	if (OpenPlugins() == -1) {
 		return 1;
 	}
@@ -197,7 +174,17 @@ file_name++;
 		return 0;
 	}
 
-	psxCpu->Execute();
+	pl_timing_prepare(Config.PsxType);
+
+	while (1)
+	{
+		stop = 0;
+		emu_action = SACTION_NONE;
+
+		psxCpu->Execute();
+		if (emu_action != SACTION_NONE)
+			do_emu_action();
+	}
 
 	return 0;
 }
