@@ -4,6 +4,9 @@
     begin                : Wed May 15 2002
     copyright            : (C) 2002 by Pete Bernert
     email                : BlackDove@addcom.de
+
+ Portions (C) Gražvydas "notaz" Ignotas, 2010-2011
+
  ***************************************************************************/
 /***************************************************************************
  *                                                                         *
@@ -20,9 +23,9 @@
 #define _IN_SPU
 
 #include "externals.h"
+#include "registers.h"
 #include "cfg.h"
 #include "dsoundoss.h"
-#include "regs.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -34,6 +37,16 @@
 #define N_(x) (x)
 #endif
 
+#ifdef __ARM_ARCH_7A__
+ #define ssat32_to_16(v) \
+  asm("ssat %0,#16,%1" : "=r" (v) : "r" (v))
+#else
+ #define ssat32_to_16(v) do { \
+  if (v < -32768) v = -32768; \
+  else if (v > 32767) v = 32767; \
+ } while (0)
+#endif
+
 /*
 #if defined (USEMACOSX)
 static char * libraryName     = N_("Mac OS X Sound");
@@ -65,7 +78,7 @@ unsigned char * pMixIrq=0;
 
 // user settings
 
-int             iVolume=3;
+int             iVolume=768; // 1024 is 1.0
 int             iXAPitch=1;
 int             iUseTimer=2;
 int             iSPUIRQWait=1;
@@ -79,7 +92,8 @@ int             iUseInterpolation=2;
 SPUCHAN         s_chan[MAXCHAN+1];                     // channel + 1 infos (1 is security for fmod handling)
 REVERBInfo      rvb;
 
-unsigned long   dwNoiseVal=1;                          // global noise generator
+unsigned int    dwNoiseVal;                            // global noise generator
+unsigned int    dwNoiseCount;
 int             iSpuAsyncWait=0;
 
 unsigned short  spuCtrl=0;                             // some vars to store psx reg infos
@@ -93,8 +107,10 @@ int             bSPUIsOpen=0;
 
 static pthread_t thread = (pthread_t)-1;               // thread id (linux)
 
-unsigned long dwNewChannel=0;                          // flags for faster testing, if new channel starts
-unsigned long dwChannelOn=0;
+unsigned int dwNewChannel=0;                           // flags for faster testing, if new channel starts
+unsigned int dwChannelOn=0;                            // not silent channels
+unsigned int dwPendingChanOff=0;
+unsigned int dwChannelDead=0;                          // silent+not useful channels
 
 void (CALLBACK *irqCallback)(void)=0;                  // func of main emu, called on spu irq
 void (CALLBACK *cddavCallback)(unsigned short,unsigned short)=0;
@@ -106,7 +122,8 @@ static const int f[8][2] = {   {    0,  0  },
                         {  115, -52 },
                         {   98, -55 },
                         {  122, -60 } };
-int SSumLR[NSSIZE*2];
+int ChanBuf[NSSIZE+3];
+int SSumLR[(NSSIZE+3)*2];
 int iFMod[NSSIZE];
 int iCycle = 0;
 short * pS;
@@ -115,6 +132,8 @@ int lastch=-1;             // last channel processed on spu irq in timer mode
 static int lastns=0;       // last ns pos
 static int iSecureStart=0; // secure start counter
 
+#define CDDA_BUFFER_SIZE (16384 * sizeof(uint32_t)) // must be power of 2
+
 ////////////////////////////////////////////////////////////////////////
 // CODE AREA
 ////////////////////////////////////////////////////////////////////////
@@ -203,9 +222,10 @@ INLINE void InterpolateUp(int ch)
    s_chan[ch].SB[32]=0;
 
    s_chan[ch].SB[28]=(s_chan[ch].SB[28]*s_chan[ch].sinc)/0x20000L;
-   if(s_chan[ch].sinc<=0x8000)
-        s_chan[ch].SB[29]=s_chan[ch].SB[30]-(s_chan[ch].SB[28]*((0x10000/s_chan[ch].sinc)-1));
-   else s_chan[ch].SB[29]+=s_chan[ch].SB[28];
+   //if(s_chan[ch].sinc<=0x8000)
+   //     s_chan[ch].SB[29]=s_chan[ch].SB[30]-(s_chan[ch].SB[28]*((0x10000/s_chan[ch].sinc)-1));
+   //else
+   s_chan[ch].SB[29]+=s_chan[ch].SB[28];
   }
  else                                                  // no flags? add bigger val (if possible), calc smaller step, set flag1
   s_chan[ch].SB[29]+=s_chan[ch].SB[28];
@@ -251,8 +271,8 @@ INLINE void StartSound(int ch)
  //s_chan[ch].bStop=0;
  //s_chan[ch].bOn=1;
 
- s_chan[ch].s_1=0;                                     // init mixing vars
- s_chan[ch].s_2=0;
+ s_chan[ch].SB[26]=0;                                  // init mixing vars
+ s_chan[ch].SB[27]=0;
  s_chan[ch].iSBPos=28;
 
  s_chan[ch].SB[29]=0;                                  // init our interpolation helpers
@@ -269,79 +289,33 @@ INLINE void StartSound(int ch)
 // ALL KIND OF HELPERS
 ////////////////////////////////////////////////////////////////////////
 
-INLINE void VoiceChangeFrequency(int ch)
+INLINE int FModChangeFrequency(int ch,int ns)
 {
- s_chan[ch].iUsedFreq=s_chan[ch].iActFreq;             // -> take it and calc steps
- s_chan[ch].sinc=s_chan[ch].iRawPitch<<4;
- if(!s_chan[ch].sinc) s_chan[ch].sinc=1;
- if(iUseInterpolation==1) s_chan[ch].SB[32]=1;         // -> freq change in simle imterpolation mode: set flag
-}
-
-////////////////////////////////////////////////////////////////////////
-
-INLINE void FModChangeFrequency(int ch,int ns)
-{
- int NP=s_chan[ch].iRawPitch;
+ unsigned int NP=s_chan[ch].iRawPitch;
+ int sinc;
 
  NP=((32768L+iFMod[ns])*NP)/32768L;
 
  if(NP>0x3fff) NP=0x3fff;
  if(NP<0x1)    NP=0x1;
 
- NP=(44100L*NP)/(4096L);                               // calc frequency
-
- s_chan[ch].iActFreq=NP;
- s_chan[ch].iUsedFreq=NP;
- s_chan[ch].sinc=(((NP/10)<<16)/4410);
- if(!s_chan[ch].sinc) s_chan[ch].sinc=1;
+ sinc=NP<<4;                                           // calc frequency
  if(iUseInterpolation==1)                              // freq change in simple interpolation mode
- s_chan[ch].SB[32]=1;
+  s_chan[ch].SB[32]=1;
  iFMod[ns]=0;
+
+ return sinc;
 }                    
 
 ////////////////////////////////////////////////////////////////////////
 
-// noise handler... just produces some noise data
-// surely wrong... and no noise frequency (spuCtrl&0x3f00) will be used...
-// and sometimes the noise will be used as fmod modulation... pfff
-
-INLINE int iGetNoiseVal(int ch)
-{
- int fa;
-
- if((dwNoiseVal<<=1)&0x80000000L)
-  {
-   dwNoiseVal^=0x0040001L;
-   fa=((dwNoiseVal>>2)&0x7fff);
-   fa=-fa;
-  }
- else fa=(dwNoiseVal>>2)&0x7fff;
-
- // mmm... depending on the noise freq we allow bigger/smaller changes to the previous val
- fa=s_chan[ch].iOldNoise+((fa-s_chan[ch].iOldNoise)/((0x001f-((spuCtrl&0x3f00)>>9))+1));
- if(fa>32767L)  fa=32767L;
- if(fa<-32767L) fa=-32767L;              
- s_chan[ch].iOldNoise=fa;
-
- if(iUseInterpolation<2)                               // no gauss/cubic interpolation?
- s_chan[ch].SB[29] = fa;                               // -> store noise val in "current sample" slot
- return fa;
-}                                 
-
-////////////////////////////////////////////////////////////////////////
-
 INLINE void StoreInterpolationVal(int ch,int fa)
 {
  if(s_chan[ch].bFMod==2)                               // fmod freq channel
   s_chan[ch].SB[29]=fa;
  else
   {
-   if((spuCtrl&0x4000)==0) fa=0;                       // muted?
-   else                                                // else adjust
-    {
-     if(fa>32767L)  fa=32767L;
-     if(fa<-32767L) fa=-32767L;              
-    }
+   ssat32_to_16(fa);
 
    if(iUseInterpolation>=2)                            // gauss/cubic interpolation
     {     
@@ -365,7 +339,7 @@ INLINE void StoreInterpolationVal(int ch,int fa)
 
 ////////////////////////////////////////////////////////////////////////
 
-INLINE int iGetInterpolationVal(int ch)
+INLINE int iGetInterpolationVal(int ch, int spos)
 {
  int fa;
 
@@ -377,7 +351,7 @@ INLINE int iGetInterpolationVal(int ch)
    case 3:                                             // cubic interpolation
     {
      long xd;int gpos;
-     xd = ((s_chan[ch].spos) >> 1)+1;
+     xd = (spos >> 1)+1;
      gpos = s_chan[ch].SB[28];
 
      fa  = gval(3) - 3*gval(2) + 3*gval(1) - gval0;
@@ -396,7 +370,7 @@ INLINE int iGetInterpolationVal(int ch)
    case 2:                                             // gauss interpolation
     {
      int vl, vr;int gpos;
-     vl = (s_chan[ch].spos >> 6) & ~3;
+     vl = (spos >> 6) & ~3;
      gpos = s_chan[ch].SB[28];
      vr=(gauss[vl]*gval0)&~2047;
      vr+=(gauss[vl+1]*gval(1))&~2047;
@@ -423,6 +397,275 @@ INLINE int iGetInterpolationVal(int ch)
  return fa;
 }
 
+static void do_irq(void)
+{
+ if(!(spuStat & STAT_IRQ))
+ {
+  spuStat |= STAT_IRQ;
+  if(irqCallback) irqCallback();
+ }
+}
+
+static void decode_block_data(int *dest, const unsigned char *src, int predict_nr, int shift_factor)
+{
+ int nSample;
+ int fa, s_1, s_2, d, s;
+
+ s_1 = dest[27];
+ s_2 = dest[26];
+
+ for (nSample = 0; nSample < 28; src++)
+ {
+  d = (int)*src;
+  s = (int)(signed short)((d & 0x0f) << 12);
+
+  fa = s >> shift_factor;
+  fa += ((s_1 * f[predict_nr][0])>>6) + ((s_2 * f[predict_nr][1])>>6);
+  s_2=s_1;s_1=fa;
+
+  dest[nSample++] = fa;
+
+  s = (int)(signed short)((d & 0xf0) << 8);
+  fa = s >> shift_factor;
+  fa += ((s_1 * f[predict_nr][0])>>6) + ((s_2 * f[predict_nr][1])>>6);
+  s_2=s_1;s_1=fa;
+
+  dest[nSample++] = fa;
+ }
+}
+
+static int decode_block(int ch)
+{
+ unsigned char *start;
+ int predict_nr,shift_factor,flags;
+ int ret = 0;
+
+ start=s_chan[ch].pCurr;                   // set up the current pos
+ if(dwPendingChanOff&(1<<ch))
+ {
+  dwChannelOn&=~(1<<ch);                   // -> turn everything off
+  dwPendingChanOff&=~(1<<ch);
+  s_chan[ch].bStop=1;
+  s_chan[ch].ADSRX.EnvelopeVol=0;
+ }
+
+ //////////////////////////////////////////// irq check
+
+ if(spuCtrl&CTRL_IRQ)
+ {
+  if(pSpuIrq == start)                     // irq address reached?
+  {
+   do_irq();                               // -> call main emu
+   ret = 1;
+  }
+ }
+
+ predict_nr=(int)start[0];
+ shift_factor=predict_nr&0xf;
+ predict_nr >>= 4;
+
+ decode_block_data(s_chan[ch].SB, start + 2, predict_nr, shift_factor);
+
+ //////////////////////////////////////////// flag handler
+
+ flags=(int)start[1];
+ if(flags&4)
+  s_chan[ch].pLoop=start;                  // loop adress
+
+ start+=16;
+ if(flags&1)                               // 1: stop/loop
+ {
+  if(!(flags&2))
+   dwPendingChanOff|=1<<ch;
+
+  start = s_chan[ch].pLoop;
+ }
+
+ if (start - spuMemC >= 0x80000) {
+  // most likely wrong
+  start = spuMemC;
+  printf("ch%d oflow\n", ch);
+ }
+
+ s_chan[ch].pCurr = start;                 // store values for next cycle
+ s_chan[ch].bJump = flags & 1;
+
+ return ret;
+}
+
+// do block, but ignore sample data
+static int skip_block(int ch)
+{
+ unsigned char *start = s_chan[ch].pCurr;
+ int flags = start[1];
+ int ret = 0;
+
+ if(start == pSpuIrq)
+ {
+  do_irq();
+  ret = 1;
+ }
+
+ if(flags & 4)
+  s_chan[ch].pLoop = start;
+
+ s_chan[ch].pCurr += 16;
+
+ if(flags & 1)
+  s_chan[ch].pCurr = s_chan[ch].pLoop;
+
+ s_chan[ch].bJump = flags & 1;
+ return ret;
+}
+
+#define make_do_samples(name, fmod_code, interp_start, interp1_code, interp2_code, interp_end) \
+static int do_samples_##name(int ch, int ns, int ns_to) \
+{                                            \
+ int sinc = s_chan[ch].sinc;                 \
+ int spos = s_chan[ch].spos;                 \
+ int sbpos = s_chan[ch].iSBPos;              \
+ int *SB = s_chan[ch].SB;                    \
+ int ret = -1;                               \
+ int d, fa;                                  \
+ interp_start;                               \
+                                             \
+ for (; ns < ns_to; ns++)                    \
+ {                                           \
+  fmod_code;                                 \
+                                             \
+  while (spos >= 0x10000)                    \
+  {                                          \
+   if(sbpos == 28)                           \
+   {                                         \
+    sbpos = 0;                               \
+    d = decode_block(ch);                    \
+    if(d && iSPUIRQWait)                     \
+    {                                        \
+     ret = ns;                               \
+     goto out;                               \
+    }                                        \
+   }                                         \
+                                             \
+   fa = SB[sbpos++];                         \
+   interp1_code;                             \
+   spos -= 0x10000;                          \
+  }                                          \
+                                             \
+  interp2_code;                              \
+  spos += sinc;                              \
+ }                                           \
+                                             \
+out:                                         \
+ s_chan[ch].sinc = sinc;                     \
+ s_chan[ch].spos = spos;                     \
+ s_chan[ch].iSBPos = sbpos;                  \
+ interp_end;                                 \
+                                             \
+ return ret;                                 \
+}
+
+#define fmod_recv_check \
+  if(s_chan[ch].bFMod==1 && iFMod[ns]) \
+    sinc = FModChangeFrequency(ch,ns)
+
+make_do_samples(default, fmod_recv_check, ,
+  StoreInterpolationVal(ch, fa),
+  ChanBuf[ns] = iGetInterpolationVal(ch, spos), )
+make_do_samples(noint, , fa = s_chan[ch].SB[29], , ChanBuf[ns] = fa, s_chan[ch].SB[29] = fa)
+
+#define simple_interp_store \
+  s_chan[ch].SB[28] = 0; \
+  s_chan[ch].SB[29] = s_chan[ch].SB[30]; \
+  s_chan[ch].SB[30] = s_chan[ch].SB[31]; \
+  s_chan[ch].SB[31] = fa; \
+  s_chan[ch].SB[32] = 1
+
+#define simple_interp_get \
+  if(sinc<0x10000)          /* -> upsampling? */ \
+       InterpolateUp(ch);   /* --> interpolate up */ \
+  else InterpolateDown(ch); /* --> else down */ \
+  ChanBuf[ns] = s_chan[ch].SB[29]
+
+make_do_samples(simple, , ,
+  simple_interp_store, simple_interp_get, )
+
+static int do_samples_noise(int ch, int ns, int ns_to)
+{
+ int level, shift, bit;
+
+ s_chan[ch].spos += s_chan[ch].sinc * (ns_to - ns);
+ while (s_chan[ch].spos >= 28*0x10000)
+ {
+  skip_block(ch);
+  s_chan[ch].spos -= 28*0x10000;
+ }
+
+ // modified from DrHell/shalma, no fraction
+ level = (spuCtrl >> 10) & 0x0f;
+ level = 0x8000 >> level;
+
+ for (; ns < ns_to; ns++)
+ {
+  dwNoiseCount += 2;
+  if (dwNoiseCount >= level)
+  {
+   dwNoiseCount -= level;
+   shift = (dwNoiseVal >> 10) & 0x1f;
+   bit = (0x69696969 >> shift) & 1;
+   if (dwNoiseVal & 0x8000)
+    bit ^= 1;
+   dwNoiseVal = (dwNoiseVal << 1) | bit;
+  }
+
+  ChanBuf[ns] = (signed short)dwNoiseVal;
+ }
+
+ return -1;
+}
+
+#ifdef __ARM_ARCH_7A__
+// asm code
+extern void mix_chan(int start, int count, int lv, int rv);
+extern void mix_chan_rvb(int start, int count, int lv, int rv);
+#else
+static void mix_chan(int start, int count, int lv, int rv)
+{
+ int *dst = SSumLR + start * 2;
+ const int *src = ChanBuf + start;
+ int l, r;
+
+ while (count--)
+  {
+   int sval = *src++;
+
+   l = (sval * lv) >> 14;
+   r = (sval * rv) >> 14;
+   *dst++ += l;
+   *dst++ += r;
+  }
+}
+
+static void mix_chan_rvb(int start, int count, int lv, int rv)
+{
+ int *dst = SSumLR + start * 2;
+ int *drvb = sRVBStart + start * 2;
+ const int *src = ChanBuf + start;
+ int l, r;
+
+ while (count--)
+  {
+   int sval = *src++;
+
+   l = (sval * lv) >> 14;
+   r = (sval * rv) >> 14;
+   *dst++ += l;
+   *dst++ += r;
+   *drvb++ += l;
+   *drvb++ += r;
+  }
+}
+#endif
+
 ////////////////////////////////////////////////////////////////////////
 // MAIN SPU FUNCTION
 // here is the main job handler... thread, timer or direct func call
@@ -440,14 +683,9 @@ INLINE int iGetInterpolationVal(int ch)
 
 static void *MAINThread(void *arg)
 {
- int s_1,s_2,fa,ns,ns_from,ns_to;
-#if !defined(_MACOSX) && !defined(__arm__)
- int voldiv = iVolume;
-#else
- const int voldiv = 2;
-#endif
- unsigned char * start;unsigned int nSample;
- int ch,predict_nr,shift_factor,flags,d,s;
+ int volmult = iVolume;
+ int ns,ns_from,ns_to;
+ int ch,d,silentch;
  int bIRQReturn=0;
 
  while(!bEndThread)                                    // until we are shutting down
@@ -484,9 +722,11 @@ static void *MAINThread(void *arg)
    ch=0;
    if(lastch>=0)                                       // will be -1 if no continue is pending
     {
-     ch=lastch; ns_from=lastns+1; lastch=-1;           // -> setup all kind of vars to continue
+     ch=lastch; ns_from=lastns; lastch=-1;             // -> setup all kind of vars to continue
     }
 
+   silentch=~(dwChannelOn|dwNewChannel);
+
    //--------------------------------------------------//
    //- main channel loop                              -// 
    //--------------------------------------------------//
@@ -496,151 +736,66 @@ static void *MAINThread(void *arg)
        if(dwNewChannel&(1<<ch)) StartSound(ch);        // start new sound
        if(!(dwChannelOn&(1<<ch))) continue;            // channel not playing? next
 
-       if(s_chan[ch].iActFreq!=s_chan[ch].iUsedFreq)   // new psx frequency?
-        VoiceChangeFrequency(ch);
-
-       for(ns=ns_from;ns<ns_to;ns++)                   // loop until 1 ms of data is reached
+       if(s_chan[ch].bNoise)
+        d=do_samples_noise(ch, ns_from, ns_to);
+       else if(s_chan[ch].bFMod==2 || (s_chan[ch].bFMod==0 && iUseInterpolation==0))
+        d=do_samples_noint(ch, ns_from, ns_to);
+       else if(s_chan[ch].bFMod==0 && iUseInterpolation==1)
+        d=do_samples_simple(ch, ns_from, ns_to);
+       else
+        d=do_samples_default(ch, ns_from, ns_to);
+       if(d>=0)
         {
-         int sval;
-
-         if(s_chan[ch].bFMod==1 && iFMod[ns])          // fmod freq channel
-          FModChangeFrequency(ch,ns);
-
-         while(s_chan[ch].spos>=0x10000L)
-          {
-           if(s_chan[ch].iSBPos==28)                   // 28 reached?
-            {
-             start=s_chan[ch].pCurr;                   // set up the current pos
-
-             if (start == (unsigned char*)-1)          // special "stop" sign
-              {
-               dwChannelOn&=~(1<<ch);                  // -> turn everything off
-               s_chan[ch].ADSRX.EnvelopeVol=0;
-               goto ENDX;                              // -> and done for this channel
-              }
-
-             s_chan[ch].iSBPos=0;
-
-             //////////////////////////////////////////// spu irq handler here? mmm... do it later
-
-             s_1=s_chan[ch].s_1;
-             s_2=s_chan[ch].s_2;
-
-             predict_nr=(int)*start;start++;
-             shift_factor=predict_nr&0xf;
-             predict_nr >>= 4;
-             flags=(int)*start;start++;
-
-             // -------------------------------------- // 
-
-             for (nSample=0;nSample<28;start++)      
-              {
-               d=(int)*start;
-               s=((d&0xf)<<12);
-               if(s&0x8000) s|=0xffff0000;
-
-               fa=(s >> shift_factor);
-               fa=fa + ((s_1 * f[predict_nr][0])>>6) + ((s_2 * f[predict_nr][1])>>6);
-               s_2=s_1;s_1=fa;
-               s=((d & 0xf0) << 8);
-
-               s_chan[ch].SB[nSample++]=fa;
-
-               if(s&0x8000) s|=0xffff0000;
-               fa=(s>>shift_factor);
-               fa=fa + ((s_1 * f[predict_nr][0])>>6) + ((s_2 * f[predict_nr][1])>>6);
-               s_2=s_1;s_1=fa;
-
-               s_chan[ch].SB[nSample++]=fa;
-              }
-
-             //////////////////////////////////////////// irq check
-
-             if(irqCallback && (spuCtrl&0x40))         // some callback and irq active?
-              {
-               if((pSpuIrq >  start-16 &&              // irq address reached?
-                   pSpuIrq <= start) ||
-                  ((flags&1) &&                        // special: irq on looping addr, when stop/loop flag is set 
-                   (pSpuIrq >  s_chan[ch].pLoop-16 &&
-                    pSpuIrq <= s_chan[ch].pLoop)))
-               {
-                 irqCallback();                        // -> call main emu
-
-                 if(iSPUIRQWait)                       // -> option: wait after irq for main emu
-                  {
-                   iSpuAsyncWait=1;
-                   bIRQReturn=1;
-                   lastch=ch; 
-                   lastns=ns;
-                   ns_to=ns+1;
-                  }
-                }
-              }
-
-             //////////////////////////////////////////// flag handler
-
-             if((flags&4) && (!s_chan[ch].bIgnoreLoop))
-              s_chan[ch].pLoop=start-16;               // loop adress
-
-             if(flags&1)                               // 1: stop/loop
-              {
-               // We play this block out first...
-               //if(!(flags&2))                          // 1+2: do loop... otherwise: stop
-               if(flags!=3 || s_chan[ch].pLoop==NULL)  // PETE: if we don't check exactly for 3, loop hang ups will happen (DQ4, for example)
-                {                                      // and checking if pLoop is set avoids crashes, yeah
-                 start = (unsigned char*)-1;
-                }
-               else
-                {
-                 start = s_chan[ch].pLoop;
-                }
-              }
-
-             s_chan[ch].pCurr=start;                   // store values for next cycle
-             s_chan[ch].s_1=s_1;
-             s_chan[ch].s_2=s_2;
-            }
-
-           fa=s_chan[ch].SB[s_chan[ch].iSBPos++];      // get sample data
-
-           StoreInterpolationVal(ch,fa);               // store val for later interpolation
-
-           s_chan[ch].spos -= 0x10000L;
-          }
-
-         if(s_chan[ch].bNoise)
-              fa=iGetNoiseVal(ch);                     // get noise val
-         else fa=iGetInterpolationVal(ch);             // get sample val
-
-         sval = (MixADSR(ch) * fa) / 1023;  // mix adsr
-
-         if(s_chan[ch].bFMod==2)                       // fmod freq channel
-          iFMod[ns]=sval;                              // -> store 1T sample data, use that to do fmod on next channel
-         else                                          // no fmod freq channel
-          {
-           //////////////////////////////////////////////
-           // ok, left/right sound volume (psx volume goes from 0 ... 0x3fff)
-
-           SSumLR[ns*2]  +=(sval*s_chan[ch].iLeftVolume)/0x4000L;
-           SSumLR[ns*2+1]+=(sval*s_chan[ch].iRightVolume)/0x4000L;
-
-           //////////////////////////////////////////////
-           // now let us store sound data for reverb    
-
-           if(s_chan[ch].bRVBActive) StoreREVERB(ch,ns,sval);
-          }
-
-         ////////////////////////////////////////////////
-         // ok, go on until 1 ms data of this channel is collected
-
-         s_chan[ch].spos += s_chan[ch].sinc;
+         bIRQReturn=1;
+         lastch=ch; 
+         lastns=ns_to=d;
+         if(d==0)
+          break;
         }
-ENDX:   ;
+
+       MixADSR(ch, ns_from, ns_to);
+
+       if(s_chan[ch].bFMod==2)                         // fmod freq channel
+        memcpy(iFMod, ChanBuf, sizeof(iFMod));
+       else if(s_chan[ch].bRVBActive)
+        mix_chan_rvb(ns_from,ns_to-ns_from,s_chan[ch].iLeftVolume,s_chan[ch].iRightVolume);
+       else
+        mix_chan(ns_from,ns_to-ns_from,s_chan[ch].iLeftVolume,s_chan[ch].iRightVolume);
       }
     }
 
-    if(bIRQReturn)                            // special return for "spu irq - wait for cpu action"
+    // advance "stopped" channels that can cause irqs
+    // (all chans are always playing on the real thing..)
+    if(!bIRQReturn && (spuCtrl&CTRL_IRQ))
+     for(ch=0;ch<MAXCHAN;ch++)
+      {
+       if(!(silentch&(1<<ch))) continue;               // already handled
+       if(dwChannelDead&(1<<ch)) continue;
+       if(s_chan[ch].pCurr > pSpuIrq && s_chan[ch].pLoop > pSpuIrq)
+        continue;
+
+       s_chan[ch].spos += s_chan[ch].sinc * NSSIZE;
+       while(s_chan[ch].spos >= 28 * 0x10000)
+        {
+         unsigned char *start = s_chan[ch].pCurr;
+
+         // no need for bIRQReturn since the channel is silent
+         iSpuAsyncWait |= skip_block(ch);
+         if(start == s_chan[ch].pCurr)
+          {
+           // looping on self
+           dwChannelDead |= 1<<ch;
+           s_chan[ch].spos = 0;
+           break;
+          }
+
+         s_chan[ch].spos -= 28 * 0x10000;
+        }
+      }
+
+    if(bIRQReturn && iSPUIRQWait)                      // special return for "spu irq - wait for cpu action"
      {
+      iSpuAsyncWait=1;
       bIRQReturn=0;
       if(iUseTimer!=2)
        { 
@@ -668,19 +823,26 @@ ENDX:   ;
   ///////////////////////////////////////////////////////
   // mix all channels (including reverb) into one buffer
 
+  if(iUseReverb)
+   REVERBDo();
+
+  if((spuCtrl&0x4000)==0) // muted? (rare, don't optimize for this)
+   {
+    memset(pS, 0, NSSIZE * 2 * sizeof(pS[0]));
+    pS += NSSIZE*2;
+   }
+  else
   for (ns = 0; ns < NSSIZE*2; )
    {
-    SSumLR[ns] += MixREVERBLeft(ns/2);
-
-    d = SSumLR[ns] / voldiv; SSumLR[ns] = 0;
-    if (d < -32767) d = -32767; if (d > 32767) d = 32767;
+    d = SSumLR[ns]; SSumLR[ns] = 0;
+    d = d * volmult >> 10;
+    ssat32_to_16(d);
     *pS++ = d;
     ns++;
 
-    SSumLR[ns] += MixREVERBRight();
-
-    d = SSumLR[ns] / voldiv; SSumLR[ns] = 0;
-    if(d < -32767) d = -32767; if(d > 32767) d = 32767;
+    d = SSumLR[ns]; SSumLR[ns] = 0;
+    d = d * volmult >> 10;
+    ssat32_to_16(d);
     *pS++ = d;
     ns++;
    }
@@ -703,7 +865,7 @@ ENDX:   ;
   // an IRQ. Only problem: the "wait for cpu" option is kinda hard to do here
   // in some of Peops timer modes. So: we ignore this option here (for now).
 
-  if(pMixIrq && irqCallback)
+  if(pMixIrq)
    {
     for(ns=0;ns<NSSIZE;ns++)
      {
@@ -712,7 +874,7 @@ ENDX:   ;
         for(ch=0;ch<4;ch++)
          {
           if(pSpuIrq>=pMixIrq+(ch*0x400) && pSpuIrq<pMixIrq+(ch*0x400)+2)
-           irqCallback();
+           do_irq();
          }
        }
       pMixIrq+=2;if(pMixIrq>spuMemC+0x3ff) pMixIrq=spuMemC;
@@ -723,7 +885,7 @@ ENDX:   ;
 
   // feed the sound
   // wanna have around 1/60 sec (16.666 ms) updates
-  if (iCycle++ > 16)
+  if (iCycle++ > 16/FRAG_MSECS)
    {
     SoundFeedStreamData((unsigned char *)pSpuBuffer,
                         ((unsigned char *)pS) - ((unsigned char *)pSpuBuffer));
@@ -742,12 +904,14 @@ ENDX:   ;
 // SPU ASYNC... even newer epsxe func
 //  1 time every 'cycle' cycles... harhar
 
+// rearmed: called every 2ms now
+
 void CALLBACK SPUasync(unsigned long cycle)
 {
  if(iSpuAsyncWait)
   {
    iSpuAsyncWait++;
-   if(iSpuAsyncWait<=16) return;
+   if(iSpuAsyncWait<=16/2) return;
    iSpuAsyncWait=0;
   }
 
@@ -789,12 +953,12 @@ void CALLBACK SPUplayADPCMchannel(xa_decode_t *xap)
 }
 
 // CDDA AUDIO
-void CALLBACK SPUplayCDDAchannel(short *pcm, int nbytes)
+int CALLBACK SPUplayCDDAchannel(short *pcm, int nbytes)
 {
- if (!pcm)      return;
- if (nbytes<=0) return;
+ if (!pcm)      return -1;
+ if (nbytes<=0) return -1;
 
- FeedCDDA((unsigned char *)pcm, nbytes);
+ return FeedCDDA((unsigned char *)pcm, nbytes);
 }
 
 // SETUPTIMER: init of certain buffers and threads/timers
@@ -852,7 +1016,7 @@ void SetupStreams(void)
  XAFeed  = XAStart;
 
  CDDAStart =                                           // alloc cdda buffer
-  (uint32_t *)malloc(16384 * sizeof(uint32_t));
+  (uint32_t *)malloc(CDDA_BUFFER_SIZE);
  CDDAEnd   = CDDAStart + 16384;
  CDDAPlay  = CDDAStart;
  CDDAFeed  = CDDAStart;
@@ -864,7 +1028,6 @@ void SetupStreams(void)
 //   s_chan[i].hMutex=CreateMutex(NULL,FALSE,NULL);
    s_chan[i].ADSRX.SustainLevel = 0xf;                 // -> init sustain
    s_chan[i].pLoop=spuMemC;
-   s_chan[i].pStart=spuMemC;
    s_chan[i].pCurr=spuMemC;
   }
 
@@ -893,8 +1056,6 @@ long CALLBACK SPUinit(void)
  memset((void *)&rvb, 0, sizeof(REVERBInfo));
  InitADSR();
 
- iVolume = 3;
- iReverbOff = -1;
  spuIrq = 0;
  spuAddr = 0xffffffff;
  bEndThread = 0;
@@ -1010,4 +1171,27 @@ char * SPUgetLibInfos(void)
 }
 */
 
+// debug
+void spu_get_debug_info(int *chans_out, int *run_chans, int *fmod_chans_out, int *noise_chans_out)
+{
+ int ch = 0, fmod_chans = 0, noise_chans = 0, irq_chans = 0;
+
+ for(;ch<MAXCHAN;ch++)
+ {
+  if (!(dwChannelOn & (1<<ch)))
+   continue;
+  if (s_chan[ch].bFMod == 2)
+   fmod_chans |= 1 << ch;
+  if (s_chan[ch].bNoise)
+   noise_chans |= 1 << ch;
+  if((spuCtrl&CTRL_IRQ) && s_chan[ch].pCurr <= pSpuIrq && s_chan[ch].pLoop <= pSpuIrq)
+   irq_chans |= 1 << ch;
+ }
+
+ *chans_out = dwChannelOn;
+ *run_chans = ~dwChannelOn & ~dwChannelDead & irq_chans;
+ *fmod_chans_out = fmod_chans;
+ *noise_chans_out = noise_chans;
+}
+
 // vim:shiftwidth=1:expandtab
