@@ -61,11 +61,6 @@
 
 #include "gpuStdafx.h"
 #endif
-#include "X11/Xlib.h"
-#include "X11/Xutil.h"
-#include "X11/Xatom.h"
-
-extern Display*		x11Display;
 
 extern void ProcessEvents();
                
@@ -547,10 +542,10 @@ void ChangeDesktop()                                   // change destop resoluti
 ////////////////////////////////////////////////////////////////////////
 
 HMENU hPSEMenu=NULL;
-void maemoGLinit();
+
 long CALLBACK GPUopen(HWND hwndGPU)                    
 #else
-long CALLBACK GPUopen(unsigned long * disp,char * CapText,char * CfgFile)
+long CALLBACK GPUopen(int hwndGPU)
 #endif
 {
 	#ifdef _WINDOWS
@@ -566,7 +561,7 @@ long CALLBACK GPUopen(unsigned long * disp,char * CapText,char * CfgFile)
 	 iResX=240;iResY=320;
 	#endif
 #ifdef MAEMO_CHANGES
-	  iResX=800;iResY=600;
+	  iResX=800;iResY=480;
 #endif
 	 iColDepth=8;
 	 bChangeRes=FALSE;
@@ -583,7 +578,7 @@ long CALLBACK GPUopen(unsigned long * disp,char * CapText,char * CfgFile)
 	 bUseFrameLimit=FALSE;
 	 bUseFrameSkip=FALSE;
 	 iFrameLimit=0;
-	 fFrameRate=60.0f;
+	 fFrameRate=50.0f;
 	 iOffscreenDrawing=0;
 	 //bOpaquePass=FALSE;
 	 //bUseAntiAlias=FALSE;
@@ -681,15 +676,9 @@ long CALLBACK GPUopen(unsigned long * disp,char * CapText,char * CfgFile)
 // lGPUstatusRet = 0x74000000;
 
 // with some emus, we could do the OGL init right here... oh my
-//
-
-GLinitialize();
- 
-
-	 
-*disp=(unsigned long)x11Display;  
-return 0;
+// if(bIsFirstFrame) GLinitialize();
 
+ return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////
