@@ -24,25 +24,29 @@
 #include "debug.h"
 
 int skipCount = 2; /* frame skip (0,1,2,3...) */
-int skCount=0; /* internal frame skip */
+int skCount = 0; /* internal frame skip */
 int linesInterlace = 0;  /* internal lines interlace */
 int linesInterlace_user = 0; /* Lines interlace */
 
-bool isSkip=false; /* skip frame (info coming from GPU) */
-bool skipFrame=false; /* skip frame (according to frame skip) */
+bool isSkip = false; /* skip frame (info coming from GPU) */
+bool wasSkip = false;
+bool skipFrame = false; /* skip frame (according to frame skip) */
 bool alt_fps = false; /* Alternative FPS algorithm */
 bool show_fps = false; /* Show FPS statistics */
 
-bool isPAL=false; /* PAL video timing */
+bool isPAL = false; /* PAL video timing */
 bool progressInterlace_flag = false; /* Progressive interlace flag */
 bool progressInterlace = false; /* Progressive interlace option*/
 bool frameLimit = false; /* frames to wait */
+
 bool light = true; /* lighting */
 bool blend = true; /* blending */
-
+bool FrameToRead = false; /* load image in progress */
+bool FrameToWrite = false; /* store image in progress */
 bool fb_dirty = false;
 
 bool enableAbbeyHack = false; /* Abe's Odyssey hack */
+
 u8 BLEND_MODE;
 u8 TEXT_MODE;
 u8 Masking;
@@ -60,9 +64,6 @@ s32		px,py;
 s32		x_end,y_end;
 u16*  pvram;
 
-u32 FrameToRead;
-u32 FrameToWrite;
-
 u32 GP0;
 s32 PacketCount;
 s32 PacketIndex;
@@ -95,7 +96,7 @@ u32   tInc, tMsk;
 
 GPUPacket PacketBuffer;
 // FRAME_BUFFER_SIZE is defined in bytes; 512K is guard memory for out of range reads
-u16   GPU_FrameBuffer[(FRAME_BUFFER_SIZE+512*1024)/2] __attribute__((aligned(16)));
+u16   GPU_FrameBuffer[(FRAME_BUFFER_SIZE+512*1024)/2] __attribute__((aligned(2048)));
 u32   GPU_GP1;
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -284,7 +285,7 @@ void  GPU_writeDataMem(u32* dmaAddress, s32 dmaCount)
 					pvram += 1024;
 					if (++py>=y_end) 
 					{
-						FrameToWrite = 0;
+						FrameToWrite = false;
 						GPU_GP1 &= ~0x08000000;
 						break;
 					}
@@ -297,7 +298,7 @@ void  GPU_writeDataMem(u32* dmaAddress, s32 dmaCount)
 					pvram += 1024;
 					if (++py>=y_end) 
 					{
-						FrameToWrite = 0;
+						FrameToWrite = false;
 						GPU_GP1 &= ~0x08000000;
 						break;
 					}
@@ -331,7 +332,7 @@ INLINE int CheckForEndlessLoop(u32 *laddr)
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-void  GPU_dmaChain(u32* baseAddr, u32 dmaVAddr)
+long GPU_dmaChain(u32* baseAddr, u32 dmaVAddr)
 {
 #ifdef DEBUG_ANALYSIS
 	dbg_anacnt_GPU_dmaChain++;
@@ -339,6 +340,7 @@ void  GPU_dmaChain(u32* baseAddr, u32 dmaVAddr)
 	pcsx4all_prof_start_with_pause(PCSX4ALL_PROF_GPU,PCSX4ALL_PROF_HW_WRITE);
 	u32 data, *address, count, offset;
 	unsigned int DMACommandCounter = 0;
+	long dma_words = 0;
 
 	GPU_GP1 &= ~0x14000000;
 	lUsedAddr[0]=lUsedAddr[1]=lUsedAddr[2]=(u32*)0x1fffff;
@@ -355,9 +357,12 @@ void  GPU_dmaChain(u32* baseAddr, u32 dmaVAddr)
 		else dmaVAddr = 0x1FFFFF;
 
 		if(count>0) GPU_writeDataMem(address,count);
+		dma_words += 1 + count;
 	}
 	GPU_GP1 = (GPU_GP1 | 0x14000000) & ~0x60000000;
 	pcsx4all_prof_end_with_resume(PCSX4ALL_PROF_GPU,PCSX4ALL_PROF_HW_WRITE);
+
+	return dma_words;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -381,7 +386,7 @@ void  GPU_writeData(u32 data)
 			pvram += 1024;
 			if (++py>=y_end) 
 			{
-				FrameToWrite = 0;
+				FrameToWrite = false;
 				GPU_GP1 &= ~0x08000000;
 			}
 		}
@@ -395,7 +400,7 @@ void  GPU_writeData(u32 data)
 				pvram += 1024;
 				if (++py>=y_end) 
 				{
-					FrameToWrite = 0;
+					FrameToWrite = false;
 					GPU_GP1 &= ~0x08000000;
 				}
 			}
@@ -448,7 +453,7 @@ void  GPU_readDataMem(u32* dmaAddress, s32 dmaCount)
 			pvram += 1024;
 			if (++py>=y_end) 
 			{
-				FrameToRead = 0;
+				FrameToRead = false;
 				GPU_GP1 &= ~0x08000000;
 				break;
 			}
@@ -481,7 +486,7 @@ u32  GPU_readData(void)
 			pvram += 1024;
 			if (++py>=y_end) 
 			{
-				FrameToRead = 0;
+				FrameToRead = false;
 				GPU_GP1 &= ~0x08000000;
 			}
 		}
@@ -493,7 +498,7 @@ u32  GPU_readData(void)
 			pvram +=1024;
 			if (++py>=y_end) 
 			{
-				FrameToRead = 0;
+				FrameToRead = false;
 				GPU_GP1 &= ~0x08000000;
 			}
 		}
@@ -529,11 +534,11 @@ void  GPU_writeStatus(u32 data)
 		break;
 	case 0x01:
 		GPU_GP1 &= ~0x08000000;
-		PacketCount = FrameToRead = FrameToWrite = 0;
+		PacketCount = 0; FrameToRead = FrameToWrite = false;
 		break;
 	case 0x02:
 		GPU_GP1 &= ~0x08000000;
-		PacketCount = FrameToRead = FrameToWrite = 0;
+		PacketCount = 0; FrameToRead = FrameToWrite = false;
 		break;
 	case 0x03:
 		GPU_GP1 = (GPU_GP1 & ~0x00800000) | ((data & 1) << 23);
@@ -547,6 +552,11 @@ void  GPU_writeStatus(u32 data)
 		DisplayArea[0] = (data & 0x000003FF); //(short)(data & 0x3ff);
 		DisplayArea[1] = ((data & 0x0007FC00)>>10); //(data & 0x000FFC00) >> 10; //(short)((data>>10)&0x1ff);
 		fb_dirty = true;
+		wasSkip = isSkip;
+		if (isSkip)
+			isSkip = false;
+		else
+			isSkip = skipFrame;
 		break;
 	case 0x07:
 		DisplayArea[4] = data & 0x000003FF; //(short)(data & 0x3ff);
@@ -634,15 +644,16 @@ INLINE void gpuVideoOutput(void)
 	int incY = (h0==480) ? 2 : 1;
 	h0=(h0==480 ? 2048 : 1024);
 
-	if (!progressInterlace)
 	{
 		const int li=linesInterlace;
+		bool pi=progressInterlace;
+		bool pif=progressInterlace_flag;
 		switch ( w0 )
 		{
 			case 256:
 				for(int y1=y0+h1; y0<y1; y0+=incY)
 				{
-					if( 0 == (y0&li) ) GPU_BlitWWDWW(	src_screen16,	dest_screen16, isRGB24);
+					if(( 0 == (y0&li) ) && ((!pi) || (pif=!pif))) GPU_BlitWWDWW(	src_screen16,	dest_screen16, isRGB24);
 					dest_screen16 += VIDEO_WIDTH;
 					src_screen16  += h0;
 				}
@@ -650,7 +661,7 @@ INLINE void gpuVideoOutput(void)
 			case 368:
 				for(int y1=y0+h1; y0<y1; y0+=incY)
 				{
-					if( 0 == (y0&li) ) GPU_BlitWWWWWWWWS(	src_screen16,	dest_screen16, isRGB24, 4);
+					if(( 0 == (y0&li) ) && ((!pi) || (pif=!pif))) GPU_BlitWWWWWWWWS(	src_screen16,	dest_screen16, isRGB24, 4);
 					dest_screen16 += VIDEO_WIDTH;
 					src_screen16  += h0;
 				}
@@ -658,7 +669,7 @@ INLINE void gpuVideoOutput(void)
 			case 320:
 				for(int y1=y0+h1; y0<y1; y0+=incY)
 				{
-					if( 0 == (y0&li) ) GPU_BlitWW(	src_screen16,	dest_screen16, isRGB24);
+					if(( 0 == (y0&li) ) && ((!pi) || (pif=!pif))) GPU_BlitWW(	src_screen16,	dest_screen16, isRGB24);
 					dest_screen16 += VIDEO_WIDTH;
 					src_screen16  += h0;
 				}
@@ -666,7 +677,7 @@ INLINE void gpuVideoOutput(void)
 			case 384:
 				for(int y1=y0+h1; y0<y1; y0+=incY)
 				{
-					if( 0 == (y0&li) ) GPU_BlitWWWWWS(	src_screen16,	dest_screen16, isRGB24);
+					if(( 0 == (y0&li) ) && ((!pi) || (pif=!pif))) GPU_BlitWWWWWS(	src_screen16,	dest_screen16, isRGB24);
 					dest_screen16 += VIDEO_WIDTH;
 					src_screen16  += h0;
 				}
@@ -674,7 +685,7 @@ INLINE void gpuVideoOutput(void)
 			case 512:
 				for(int y1=y0+h1; y0<y1; y0+=incY)
 				{
-					if( 0 == (y0&li) ) GPU_BlitWWSWWSWS(	src_screen16, dest_screen16, isRGB24);
+					if(( 0 == (y0&li) ) && ((!pi) || (pif=!pif))) GPU_BlitWWSWWSWS(	src_screen16, dest_screen16, isRGB24);
 					dest_screen16 += VIDEO_WIDTH;
 					src_screen16  += h0;
 				}
@@ -682,69 +693,13 @@ INLINE void gpuVideoOutput(void)
 			case 640:
 				for(int y1=y0+h1; y0<y1; y0+=incY)
 				{
-					if( 0 == (y0&li) ) GPU_BlitWS(	src_screen16, dest_screen16, isRGB24);
+					if(( 0 == (y0&li) ) && ((!pi) || (pif=!pif))) GPU_BlitWS(	src_screen16, dest_screen16, isRGB24);
 					dest_screen16 += VIDEO_WIDTH;
 					src_screen16  += h0;
 				}
 				break;
 		}
-	}
-	else
-	{
-		const int li=linesInterlace;
-		bool flag=progressInterlace_flag; /* progressive interlace */
-		switch ( w0 )
-		{
-			case 256:
-				for(int y1=y0+h1; y0<y1; y0+=incY)
-				{
-					if(( 0 == (y0&li) ) && (flag=!flag)) GPU_BlitWWDWW(	src_screen16,	dest_screen16, isRGB24);
-					dest_screen16 += VIDEO_WIDTH;
-					src_screen16  += h0;
-				}
-				break;
-			case 368:
-				for(int y1=y0+h1; y0<y1; y0+=incY)
-				{
-					if(( 0 == (y0&li) ) && (flag=!flag)) GPU_BlitWWWWWWWWS(	src_screen16,	dest_screen16, isRGB24, 4);
-					dest_screen16 += VIDEO_WIDTH;
-					src_screen16  += h0;
-				}
-				break;
-			case 320:
-				for(int y1=y0+h1; y0<y1; y0+=incY)
-				{
-					if(( 0 == (y0&li) ) && (flag=!flag)) GPU_BlitWW(	src_screen16,	dest_screen16, isRGB24);
-					dest_screen16 += VIDEO_WIDTH;
-					src_screen16  += h0;
-				}
-				break;
-			case 384:
-				for(int y1=y0+h1; y0<y1; y0+=incY)
-				{
-					if(( 0 == (y0&li) ) && (flag=!flag)) GPU_BlitWWWWWS(	src_screen16,	dest_screen16, isRGB24);
-					dest_screen16 += VIDEO_WIDTH;
-					src_screen16  += h0;
-				}
-				break;
-			case 512:
-				for(int y1=y0+h1; y0<y1; y0+=incY)
-				{
-					if(( 0 == (y0&li) ) && (flag=!flag)) GPU_BlitWWSWWSWS(	src_screen16, dest_screen16, isRGB24);
-					dest_screen16 += VIDEO_WIDTH;
-					src_screen16  += h0;
-				}
-				break;
-			case 640:
-				for(int y1=y0+h1; y0<y1; y0+=incY)
-				{
-					if(( 0 == (y0&li) ) && (flag=!flag)) GPU_BlitWS(	src_screen16, dest_screen16, isRGB24);
-					dest_screen16 += VIDEO_WIDTH;
-					src_screen16  += h0;
-				}
-				break;
-		}
-		progressInterlace_flag=!flag;
+		progressInterlace_flag=!progressInterlace_flag;
 	}
 	video_flip();
 }
@@ -793,7 +748,7 @@ void  GPU_updateLace(void)
 	GPU_GP1 ^= 0x80000000;
 
 	// Update display
-	if ((!skipFrame) && (!isSkip) && (!(((GPU_GP1&0x08000000))||((GPU_GP1&0x00800000)))))
+	if ((!skipFrame) && (!isSkip) && (fb_dirty) && (!(((GPU_GP1&0x08000000))||((GPU_GP1&0x00800000)))))
 	{
 		gpuVideoOutput(); // Display updated
 
@@ -807,18 +762,18 @@ void  GPU_updateLace(void)
 			linesInterlace = linesInterlace_user; // resolution changed from 480 to lower one
 			video_clear();
 		}
+	}
 
-		// Limit FPS
-		if (frameLimit)
+	// Limit FPS
+	if (frameLimit)
+	{
+		static unsigned next=get_ticks();
+		if (!skipFrame)
 		{
-			static unsigned next=get_ticks();
 			unsigned now=get_ticks();
-			if (!skipFrame)
-			{
-				if (now<next) wait_ticks(next-now);
-			}
-			next+=(isPAL?(1000000/50):((unsigned)(1000000.0/59.94)));
+			if (now<next) wait_ticks(next-now);
 		}
+		next+=(isPAL?(1000000/50):((unsigned)(1000000.0/59.94)));
 	}
 
 	// Show FPS statistics
@@ -826,7 +781,7 @@ void  GPU_updateLace(void)
 	{
 		static u32 real_fps=0;
 		static u32 prev=get_ticks();
-		static char msg[24]="FPS=000/00 SPD=000%";
+		static char msg[32]="FPS=000/00 SPD=000%";
 		u32 now=get_ticks();
 		real_fps++;
 		if ((now-prev)>=1000000)
@@ -857,6 +812,7 @@ void  GPU_updateLace(void)
 			isSkip=true;
 		}
 	}
+	fb_dirty=false;
 
 	pcsx4all_prof_end_with_resume(PCSX4ALL_PROF_GPU,PCSX4ALL_PROF_COUNTERS);
 }
@@ -864,22 +820,22 @@ void  GPU_updateLace(void)
 #else
 
 #include "../../frontend/plugin_lib.h"
-#include "../../frontend/arm_utils.h"
+#include "../../frontend/cspace.h"
 
 extern "C" {
 
 static const struct rearmed_cbs *cbs;
 static void *screen_buf;
+static s16 old_res_horz, old_res_vert, old_rgb24;
 
 static void blit(void)
 {
-	static s16 old_res_horz, old_res_vert, old_rgb24;
 	s16 isRGB24 = (GPU_GP1 & 0x00200000) ? 1 : 0;
 	s16 h0, x0, y0, w0, h1;
 	u16 *srcs;
 	u8  *dest;
 
-	x0 = DisplayArea[0] & ~3; // alignment needed by blitter
+	x0 = DisplayArea[0] & ~1; // alignment needed by blitter
 	y0 = DisplayArea[1];
 	srcs = &((u16*)GPU_FrameBuffer)[FRAME_OFFSET(x0,y0)];
 
@@ -897,7 +853,7 @@ static void blit(void)
 		old_res_horz = w0;
 		old_res_vert = h1;
 		old_rgb24 = (s16)isRGB24;
-		screen_buf = cbs->pl_fbdev_set_mode(w0, h1, isRGB24 ? 24 : 16);
+		screen_buf = cbs->pl_vout_set_mode(w0, h1, isRGB24 ? 24 : 16);
 	}
 	dest = (u8 *)screen_buf;
 
@@ -923,7 +879,30 @@ static void blit(void)
 		}
 	}
 
-	screen_buf = cbs->pl_fbdev_flip();
+	screen_buf = cbs->pl_vout_flip();
+}
+
+static void blit_raw(void)
+{
+	s16 isRGB24 = (GPU_GP1 & 0x00200000) ? 1 : 0;
+	s16 h0, w0, h1;
+
+	w0 = DisplayArea[2];
+	h0 = DisplayArea[3];  // video mode
+	h1 = DisplayArea[5] - DisplayArea[4]; // display needed
+	if (h0 == 480) h1 = Min2(h1*2,480);
+
+	if (h1 <= 0)
+		return;
+
+	if (w0 != old_res_horz || h1 != old_res_vert || isRGB24 != old_rgb24)
+	{
+		old_res_horz = w0;
+		old_res_vert = h1;
+		old_rgb24 = (s16)isRGB24;
+		screen_buf = cbs->pl_vout_set_mode(w0, h1, isRGB24 ? 24 : 16);
+	}
+	cbs->pl_vout_raw_flip(DisplayArea[0], DisplayArea[1]);
 }
 
 void GPU_updateLace(void)
@@ -934,27 +913,33 @@ void GPU_updateLace(void)
 	if (!fb_dirty || (GPU_GP1&0x08800000))
 		return;
 
-	if (!isSkip) {
-		blit();
-
+	if (!wasSkip) {
+		if (cbs->pl_vout_raw_flip != NULL)
+			blit_raw();
+		else
+			blit();
 		fb_dirty = false;
-		if (*cbs->fskip_option)
-			isSkip = true;
+		skCount = 0;
 	}
-	else
-		isSkip = false;
+	else {
+		skCount++;
+		if (skCount >= 8)
+			wasSkip = isSkip = 0;
+	}
+
+	skipFrame = cbs->fskip_advice || cbs->frameskip == 1;
 }
 
 long GPUopen(unsigned long *, char *, char *)
 {
-	cbs->pl_fbdev_open();
-	screen_buf = cbs->pl_fbdev_flip();
+	cbs->pl_vout_open();
+	screen_buf = cbs->pl_vout_flip();
 	return 0;
 }
 
 long GPUclose(void)
 {
-	cbs->pl_fbdev_close();
+	cbs->pl_vout_close();
 	return 0;
 }
 
@@ -968,6 +953,12 @@ long GPUfreeze(unsigned int ulGetFreezeData, GPUFreeze_t* p2)
 
 void GPUrearmedCallbacks(const struct rearmed_cbs *cbs_)
 {
+	enableAbbeyHack = cbs_->gpu_unai.abe_hack;
+	light = !cbs_->gpu_unai.no_light;
+	blend = !cbs_->gpu_unai.no_blend;
+	if (cbs_->pl_vout_set_raw_vram)
+		cbs_->pl_vout_set_raw_vram((void *)GPU_FrameBuffer);
+
 	cbs = cbs_;
 }
 
